module CURIC::DCUI
  class Collection3WH
    def self.toggle(show, definition)
      # p definition
      model_id = definition.get_attribute('3DWarehouse', 'ModelID')
      collection = model_collection(model_id)
      # p 'Collection3WH.toggle'
      # p "model_id: #{model_id}"
     
      return unless collection
      # p "collection: #{collection['id']}"

      models = get_models(collection['id'])
    end

    def self.download(entity)
      directory = Sketchup.temp_dir
      version = Sketchup.version.split('.')[0]
      download_skp_with_progressbar(entity, "s#{version}", directory) do |e, file|
        d = Sketchup.active_model.definitions.load(file)
        d.set_attribute('3DWarehouse', 'ModelID', e['id'])
        Sketchup.active_model.place_component(d)
      end
    end

    def self.download_skp_with_progressbar(entity, version = 's20', directory = nil, &block)
      entity_id = entity['id']

      binary_version = get_binary_version(entity, version)
      return unless binary_version

      original_file_name = binary_version['originalFileName']
      version = binary_version['name']

      url = "https://3dwarehouse.sketchup.com/warehouse/v1.0/entities/#{entity_id}/binaries/#{version}?download=true"
      
      if directory.nil?
        directory = UI.select_directory
        return unless directory
      end

      file_name = File.join(directory, original_file_name)

      request = Sketchup::Http::Request.new(url)
      request.set_download_progress_callback do |current, total|
        per = ((current.to_f/total)*100).round
        size = format_size(total)
        bar = '['
        bar += '.' * 100
        bar += ']'
        bar[1..per] = '|' * per
        Sketchup.status_text = "Downloading: #{original_file_name} - #{bar}"
        Sketchup.set_status_text("#{per}%/#{size}", SB_VCB_LABEL)

        if per % 10 == 0 && block_given?
          block.call(entity, file_name, false, per)
        end
      end
      
      request.start do |request, response|
        File.open(file_name, 'wb') do |file|
          file.binmode
          file.write(response.body)
          block.call(entity, file_name, true) if block_given?
        end
      end
    end

    def self.get_binary_version(entity, version = 's20')
      binary_version = entity['binaries'][version]

      unless binary_version
        versions = %w[s20 s19 s18 s17 s8]
        v_key = versions[versions.index(version)..versions.length - 1].find { |v| entity['binaries'][v] }
        binary_version = entity['binaries'][v_key] if v_key
      end

      binary_version
    end

    def self.format_size(size)
      size = "#{(size * 0.00000095367432).round(1)} Mb"
      size = "#{size.to_i * 0.0009765625} Gb" if size.to_i > 1000

      size
    end
  end
end
