# Example to tracking Dc redraw a component
# class FittingObserver
#   def onBeforeEntityRedraw(entity)
#     p "onBeforeEntityRedraw: #{entity}"
#   end
#   def onAfterEntityRedraw(entity)
#     p "onAfterEntityRedraw: #{entity}"
#   end
# end
#
# CURIC::DCUI_DCObserver.observer.remove_observer(@fitting_ob) if @fitting_ob
# @fitting_ob = FittingObserver.new
# CURIC::DCUI_DCObserver.observer.add_observer(@fitting_ob)

# Lớp chứa Observer để Notitfy khi DCUI_DCObserver redraw

Sketchup.require('su_dynamiccomponents')

module CURIC
  module DCUI_DCObserver
    class << self
      attr_reader :observer
    end

    def self.initialize_observer
      return if @observer

      @observer ||= Observer.new(self)

      initialize_tracepoint

      enable
    end

    def self.enable
      @trace_start.enable
      @trace_end.enable
    end

    def self.enable?
      @trace_start.enabled?
    end

    def self.disable
      @trace_start.disable
      @trace_end.disable
    end

    def self.initialize_tracepoint
      @trace_start = TracePoint.new(:call) do |tp|
        begin
          if tp.defined_class == DynamicComponentsV1 && tp.method_id == :redraw
            if tp.binding.eval('is_recursive_call') == false
              entity = tp.binding.eval('entity')
              CURIC::DCUI_DCObserver.observer.notify_start_redraw(entity)
            end
          end
        rescue StandardError => e
          p e
        end
      end

      @trace_end = TracePoint.new(:return) do |tp|
        next unless tp.defined_class == DynamicComponentsV1 
        # p tp.method_id

        begin
          if tp.method_id == :redraw
            progress_bar_visible = tp.binding.eval('progress_bar_visible')
            is_recursive_call = tp.binding.eval('is_recursive_call')
            if tp.binding.eval('is_recursive_call') == false
              entity = tp.binding.eval('entity')
              CURIC::DCUI_DCObserver.observer.notify_end_redraw(entity, progress_bar_visible)
            end
          elsif tp.method_id == :refresh_dialogs
            selection_just_changed = tp.binding.eval('selection_just_changed')
            CURIC::DCUI_DCObserver.observer.notify_refresh_dialogs(selection_just_changed)
          elsif tp.method_id == :rename_attribute
            entity = tp.binding.eval('entity')
            name = tp.binding.eval('name')
            new_name = tp.binding.eval('new_name')
            new_label = tp.binding.eval('new_label')
            # p "rename_attribute: #{entity}: #{name} => #{new_name} (#{new_label})"
            CURIC::DCUI_DCObserver.observer.notify_rename_attribute(entity, name, new_name, new_label)
          end
        rescue StandardError => e
          p e
        end
      end
    end

    class Observer
      attr_accessor :observers

      def initialize(dc)
        @dc_tracking = dc
        @observers = []
      end
        
      # observers
      def add_observer(*observers)
        observers.each { |observer| @observers << observer }
      end

      def remove_observer(*observers)
        observers.each { |observer| @observers.delete(observer) }
      end

      def notify_start_redraw(entity, progress_bar_visible=true)
        @observers.each { |observer| observer.onBeforeEntityRedraw(entity, progress_bar_visible) }
      end

      def notify_end_redraw(entity, progress_bar_visible)
        # p 'notify_end_redraw'
        @observers.each { |observer| observer.onAfterEntityRedraw(entity, progress_bar_visible) }
      end

      def notify_refresh_dialogs(selection_just_changed)
        # p 'notify_refresh_dialogs'
        @observers.each { |observer| observer.refreshDialogs(selection_just_changed) }
      end

      def notify_rename_attribute(entity, name, new_name, new_label)
        @observers.each { |observer|
          if observer.respond_to?(:onRenameAttribute)
            observer.onRenameAttribute(entity, name, new_name, new_label) 
          end
        }
      end
    end
  end
end

file = __FILE__
unless file_loaded?(file)
  # Khởi tạo observer
  
  unless CURIC::DCUI_DCObserver.observer
    CURIC::DCUI_DCObserver.initialize_observer

    file_loaded file
  end
end
