

module CURIC::DCUI
  # Custom Record
  module CustomAttr
    include AttributeHelper

    @path = [PLUGIN_NAME, 'CustomAttr']
    @attrs = %w[
      display_type
      display_value
      suggestions
      radiostyle
      slidermax
      slidermin
      sliderstep
      behavior
      hide_label

      legends

      set_input_width
      input_width

      tooltip

      image

      option_use_image
      option_image_size
      option_hide_text
      image_options
    ]

    module_function

    def save_tree(definition, data)
      attributes = data['attr_groups'].clone
      attributes.each { |a| save_attribute(definition, a, true) }
    end

    def save_attribute(definition, item, deep = false)
      # p "save_attribute: #{item['key']}-#{item['formlabel']}"

      data = item.clone

      data.delete('attributes')
      data.delete('legends')

      data = JSON.parse(data.to_json)
      update(definition, data)

      if deep
        if item['attributes']
          item['attributes'].each { |a| save_attribute(definition, a, deep) }
        end

        # Fieldset
        if item['legends']
          item['attributes'].each { |a| save_attribute(definition, a, deep) }
        end
      end
    end

    def get_data(definition, key)
      items = AttributeHelper.dictionary(definition, @path, false)
      json = (items && items[key]) ? items[key] : 'null'
      JSON.parse(json)
    end

    def attributes(definition)
      ad_groups = AttributeHelper.dictionary(definition, @path, false)

      items = []

      ad_groups&.each do |_id, json|
        items << JSON.parse(json)
      end

      items
    end

    def add(definition, item)
      p 'add CustomAttr'

      update(definition, item)
    end

    def edit(definition, item)
      p 'edit CustomAttr'
      update(definition, item)
    end

    def remove(definition, item)
      items = AttributeHelper.dictionary(definition, @path)
      key = item.is_a?(String) ? item : item['key'] || item[:key]
      items.delete_key(key)
    rescue StandardError => e
      p e
    end

    def rename_key(definition)
      
    end

    def update(definition, item)
      items = AttributeHelper.dictionary(definition, @path, true)

      items[item['key']] = item.to_json
    end
  end
end
