require 'sketchup.rb'

module CURIC
  class << self
    attr_accessor :tools_command
  end
  @tools_command ||= UI.menu('Tools').add_submenu('Curic')
  ICON ||= File.join(File.dirname(__FILE__), 'Resources/icon.png')
  
  module DCUI
    Sketchup.require 'curic_dcui/curic_dc_observer'
    Sketchup.require 'curic_dcui/dc'
    Sketchup.require 'curic_dcui/dc_function'
    Sketchup.require 'curic_dcui/main'
    Sketchup.require 'curic_dcui/utils'
    Sketchup.require 'curic_dcui/observer'
    Sketchup.require 'curic_dcui/tool'
    Sketchup.require 'curic_dcui/options'
    Sketchup.require 'curic_dcui/custom_attr'
    Sketchup.require 'curic_dcui/tooltip'
    
    IS_WIN = Sketchup.platform == :platform_win
    IS_OSX = Sketchup.platform == :platform_osx

    # key name
    SHIFT_KEY = IS_WIN ? 'Shift' : 'shift'
    CTRL_KEY = IS_WIN ? 'Ctrl' : 'option'
    TAB_KEY = IS_WIN ? 'Tab' : 'command'
    TAB_KEY_ID = IS_WIN ? 9 : ALT_MODIFIER_KEY

    # fix scale_factor
    SCALE_FACTOR = if Sketchup.version.to_f > 16
                     UI.scale_factor
                   else
                     1
                   end

    PATH_R = File.join(File.dirname(__FILE__), 'Resources')
    ICON = File.join(PATH_R, 'icon.png')

    DCP_ATTR_D = 'curic_dc_plus'.freeze
    DC_ATTR_D = 'dynamic_attributes'.freeze


    CURRENT_VIEW = -1

    TOP_VIEW = Layout::SketchUpModel::TOP_VIEW
    BOTTOM_VIEW = Layout::SketchUpModel::BOTTOM_VIEW
    FRONT_VIEW = Layout::SketchUpModel::FRONT_VIEW
    BACK_VIEW = Layout::SketchUpModel::BACK_VIEW
    LEFT_VIEW = Layout::SketchUpModel::LEFT_VIEW
    RIGHT_VIEW = Layout::SketchUpModel::RIGHT_VIEW

    VIEWS = {
      TOP_VIEW => 'TOP_VIEW',
      BOTTOM_VIEW => 'BOTTOM_VIEW',
      FRONT_VIEW => 'FRONT_VIEW',
      BACK_VIEW => 'BACK_VIEW',
      LEFT_VIEW => 'LEFT_VIEW',
      RIGHT_VIEW => 'RIGHT_VIEW',
      CURRENT_VIEW => 'CURRENT_VIEW'
    }.freeze

    VECTOR_VIEWS = {
      TOP_VIEW => Z_AXIS,
      BOTTOM_VIEW => Z_AXIS.reverse,
      FRONT_VIEW => Y_AXIS.reverse,
      BACK_VIEW => Y_AXIS,
      LEFT_VIEW => X_AXIS.reverse,
      RIGHT_VIEW => X_AXIS
    }.freeze

    def self.get_settings
      @settings = Settings.new(PLUGIN.inspect.gsub('::', '_').downcase)

      @settings.set_default(:debug, false)
      
      @settings.set_default(:auto_collapse_options, false)
      @settings.set_default(:time_collapse_options, 0.5)
      @settings.set_default(:dialog_options_min_w, 310)
      @settings.set_default(:dialog_options_min_h, 60)
      @settings.set_default(:dialog_options_w, 310)
      @settings.set_default(:dialog_options_h, 600)

      @settings.set_default(:show_dc_options, false)

      @settings
    end

    def self.settings
      get_settings unless @settings

      @settings
    end

    # add observers
    class << self
      attr_reader :dialog_observer
    end

    file = __FILE__
    unless file_loaded? file
      subtools = CURIC.tools_command.add_submenu(PLUGIN_ID)
      ex = IS_WIN ? 'svg' : 'pdf'
      
      cmd_options = UI::Command.new('Options') do
        CURIC::DCUI::OptionsUI.toggle_dialog
      end

      cmd_options.small_icon = File.join(PATH_R, "icon.#{ex}")
      cmd_options.large_icon = File.join(PATH_R, "icon.#{ex}")
      cmd_options.tooltip = 'Options'
      
      # Toolbar
      tb = UI::Toolbar.new(PLUGIN_NAME)
      tb.add_item(cmd_options)
      UI.start_timer(0.1, false) { tb.restore }
      
      subtools.add_item(cmd_options)
      
      @dialog_observer = DialogObserver.new

      file_loaded file
    end
  end
end
