module CURIC::DCUI
  module_function

  def select_all_instances(path)
    p "select_all_instances(#{path})"
   end
 
  def place_component(path)
    @model = Sketchup.active_model
    
    begin
      definition = @model.definitions.load(path)
    rescue StandardError => e
      definition = nil
    end

    @model.place_component(definition) if definition

    definition
  end
 
  def place_material(path)
    @model = Sketchup.active_model

    begin
      material = @model.materials.load(path)
    rescue StandardError => e
      material = nil
    end

    material
  end
 
  def replace_component(path)
    @model = Sketchup.active_model
    @selection = @model.selection

    selected_comps = @selection.grep(Sketchup::ComponentInstance)
    return unless selected_comps && !selected_comps.empty?

    begin
      definition = @model.definitions.load(path)
    rescue StandardError => e
      definition = nil
    end
    
    if definition
      replace = selected_comps.map do |ins|
        @model.active_entities.add_instance(definition, ins.transformation)
      end
      
      @model.active_entities.erase_entities(selected_comps)
      @selection.add(replace)
    end

    definition
  end
 
  # Redraw DC(s)
  def redraw
    p 'redraw'
  end

  # Is a product? Check definition attribute 'Category' & 'itemcode'
  # @param [Sketchup::ComponentInstance) || Sketchup::ComponentDefinition] entity
  def dc?(entity)
    unless entity.is_a?(Sketchup::Group) || 
        entity.is_a?(Sketchup::ComponentInstance) ||
        entity.is_a?(Sketchup::ComponentDefinition)
      return
    end

    definition = if entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
                   entity.definition
                 else
                   entity
                 end
    e_ads = entity.attribute_dictionaries
    d_ads = definition.attribute_dictionaries
    return unless e_ads || d_ads

    show = [
      'LIST',
      'TEXTBOX',
      'VIEW'
    ]

    dc = false
    if e_ads && e_ads['dynamic_attributes']
      dc_e = e_ads['dynamic_attributes']
      dc = dc_e.keys.any? {|name| show.include?(dc_e[name]) }
    end

    unless dc
      if d_ads && d_ads['dynamic_attributes']
        dc_d = d_ads['dynamic_attributes']
        dc = dc_d.keys.any? {|name| show.include?(dc_d[name]) }
      end
    end

    dc
  end
  
  # Value of dynamic component attribute
  def dc_attribute_value(entity, attribute)
    value = entity.get_attribute(DC_ATTR_D, attribute)
    if !value && !entity.is_a?(Sketchup::ComponentDefinition)
      value = entity.definition.get_attribute(DC_ATTR_D, attribute)
    end

    value
  end
  
end
