module CURIC::DCUI
  def self.attach_dialog_observers
    model = Sketchup.active_model
    selection = model.selection

    @app_observer ||= AppOb.new
    @selection_observer ||= SelectionOb.new

    detach_dialog_observers

    Sketchup.add_observer(@app_observer)
    model.add_observer(@model_observer)
    selection.add_observer(@selection_observer)
  end

  def self.detach_dialog_observers
    Sketchup.remove_observer(@app_observer)

    model = Sketchup.active_model
    model.remove_observer(@model_observer)
    model.selection.remove_observer(@selection_observer)
  end

  class DialogObserver
    attr_accessor :dc_observer

    def initialize
      Sketchup.add_observer(AppOb.new(self))
    end

    class SelectionOb < Sketchup::SelectionObserver
      def initialize(observer = nil)
        @observer = observer
      end

      def onSelectionAdded(_selection, _element)
        selection_changed
      end

      def onSelectionBulkChange(_selection)
        # p 'onSelectionBulkChange'
        selection_changed
      end

      def onSelectionCleared(_selection)
        # p 'onSelectionCleared'
        selection_changed
      end

      def onSelectionRemoved(_selection, _element)
        # p 'onSelectionRemoved'
        selection_changed
      end

      private

      def selection_changed
        OptionsUI.selection_changed
      end
    end

    # Lớp check sự thay đổi của danh sách definitions trong model
    # Phát hiện khi có 1 sản phẩm mới trong model
    class DefinitionsOb < Sketchup::DefinitionsObserver
      def initialize(observer = nil)
        @observer = observer
        model = Sketchup.active_model
        definitions = model.definitions
      end

      # Gán lớp quan sát lên definition khi 1 product được thêm vào model
      def onComponentAdded(_ds, d)
        unless d.deleted?
        end
      end
    end

    # Lớp DefinitionOb quan sát sự thay đổi instances của 1 definition
    class DefinitionOb < Sketchup::DefinitionObserver
      def initialize(observer = nil)
        @observer = observer
      end

      def onComponentInstanceAdded(definition, instance)
        return if definition.deleted?
        return if instance.deleted?

        p "Add new product member: #{definition.name}"
      end

      def onComponentInstanceRemoved(definition, _instance)
        return if definition.deleted?

        p "Remove a product member: #{definition.name}"
      end
    end

    class DC_Observer
      attr_accessor :disable, :need_redraw, :remove_after_swap

      def initialize(observer = nil)
        @disable = false
        @need_redraw = []
        @remove_after_swap = []

        @observer = observer
        @observer.dc_observer = self
      end

      def onBeforeEntityRedraw(entity, progress_bar_visible)
        # p "onBeforeEntityRedraw: #{entity}"
      end
      
      def onAfterEntityRedraw(entity, progress_bar_visible)
        return if @disable
        # return unless progress_bar_visible

        # p 'onAfterEntityRedraw'

        # p '@need_redraw'
        # ap @need_redraw.map { |e| e.definition.name }
        
        # p '@remove_after_swap'
        # p @remove_after_swap

        @remove_after_swap.each do |array|
          entity, purge = array
          next unless entity && !entity.deleted?
          
          if purge && entity.definition.count_used_instances == 1
            ents = entity.definition.entities
            ents.erase_entities(ents.to_a)
          else
            entity.parent.entities.erase_entities([entity])
          end
        end

        # p "after remove"
        # p @remove_after_swap

        # dc = $dc_observers.get_latest_class
        # @need_redraw.each { |e| dc.redraw(e) }

        @need_redraw = []
        @remove_after_swap = []

        OptionsUI.entity_redrawed(entity)
          
      rescue => exception
        p "Error Swap Component"
        p exception
      end

      def onRenameAttribute(entity, name, new_name, new_label)
        p "onRenameAttribute:"
        p [entity, name, new_name, new_label]
        OptionsUI.rename_attribute(entity, name, new_name, new_label)
      end

      def onDeleteAttribute(data)
        p "onDeleteAttribute:"
        # ap data
      end

      def refreshDialogs(selection_just_changed)
        return if selection_just_changed == true
        
        return unless OptionsUI.dialog && OptionsUI.dialog.visible?

        # p "DC.refreshDialogs: selection_just_changed = #{selection_just_changed}"
        OptionsUI.refresh_dialog()
      end

    end

    class AppOb < Sketchup::AppObserver
      def initialize(observer = nil)
        @observer = observer
        begin
          attach_observers(Sketchup.active_model)
        rescue StandardError
        end
      end

      def onNewModel(model)
        attach_observers(model)
      end

      def onOpenModel(model)
        attach_observers(model)
      end

      def onActivateModel(model)
        if OptionsUI.dialog && OptionsUI.dialog.visible?
          # OptionsUI.dialog.execute_script('saveData()')
          OptionsUI.refresh_dialog
        end
      end

      def onQuit()
        puts "onQuit"
        if OptionsUI.dialog && OptionsUI.dialog.visible?
          OptionsUI.dialog.execute_script('saveData()')
        end
      end

      private

      def attach_observers(model)
        model.selection.add_observer(SelectionOb.new(@observer))
        model.definitions.add_observer(DefinitionsOb.new(@observer))
        begin
          CURIC::DCUI_DCObserver.observer.add_observer(DC_Observer.new(@observer))
        rescue StandardError => e
          p 'Can not add_observer to CURIC::DC.observer'
          p e
        end
      end
    end
  end
end
