require 'fileutils'

module CURIC::DCUI
  module UIMethods
    def refresh_workspace
      @model = Sketchup.active_model
      @selection = @model.selection
      @materials = @model.materials
      @pages = @model.pages
      @view = @model.active_view
      
      @path = File.join(File.dirname(__FILE__), 'ui')
      @dc = $dc_observers.get_latest_class
    end
  end
  
  # Nhóm các option
  # Material with thumbnail
  
  module OptionsUI
    Sketchup.require 'curic_dcui/bridge'
    Sketchup.require 'curic_dcui/tree_ui'
    Sketchup.require 'curic_dcui/collection'
    require 'cgi'

    extend UIMethods

    class << self
      attr_reader :dialog

      # Biến lưu tạm thời các giá trị của các attrs thay đổi tren dialog chờ redraw
      attr_reader :cache_attrs, :data_attrs

      # the current list
      attr_reader :items, :type

      attr_accessor :disable_ui, :path_lib
    end
    @disable_ui = false
    @show_collection = false
    @data_attrs = {}
    @cache_attrs = {}

    def self.show_dialog
      if @dialog && @dialog.visible?
        @dialog.bring_to_front
      else
        toggle_dialog
      end
    end

    def self.close_dialog
      toggle_dialog if @dialog && @dialog.visible?
    end

    def self.toggle_dialog
      return @dialog.close if @dialog && @dialog.visible?

      # check data
      # if !PLUGIN.products || PLUGIN.products.empty?
      #   UI.messagebox('No Product data. Please try again!')
      #   PLUGIN.query_products
      # end

      @tool = DCUITool.new

      @on_check_dialog_size = false

      refresh_workspace
      d = IS_WIN ? 15 : 0
      @dialog = UI::HtmlDialog.new(
        dialog_title: 'Component Options +',
        preferences_key: "#{PLUGIN_NAME}.dialog.options",
        scrollable: true,
        resizable: true,
        width: 290,
        height: 500,
        left: 100,
        top: 100,
        # min_width: PLUGIN.settings[:dialog_options_min_w] + d,
        # min_height: PLUGIN.settings[:dialog_options_min_h] + d,
        style: UI::HtmlDialog::STYLE_UTILITY
      )

      # html = File.join(@path, 'html/graph.html')
      html = File.join(@path, 'html/options.html')
      @dialog.set_file(html)

      add_action_callback

      # remove observer when dialog on closed
      @dialog.set_on_closed {
        PLUGIN.detach_dialog_observers 
      }

      @dialog.show

      # Bridge.decorate(@dialog)
    end

    def self.mouse_focus_dialog(attribute)
      return unless @dialog && @dialog.visible?
      return unless attribute_has_tooltip?(attribute)

      if attribute['hover']
        @tool.attribute = attribute
        @model.tools.push_tool(@tool)
      else
        @model.tools.pop_tool
        @view.invalidate
      end
    end

    def self.attribute_has_tooltip?(attribute)
      %w[lenx leny lenz].include?(attribute['name'])
    end

    def self.get_fix_size
      fix_width = Sketchup.read_default(PLUGIN_NAME, 'fix_width')
      fix_height = Sketchup.read_default(PLUGIN_NAME, 'fix_height')

      [fix_width, fix_height]
    end

    def self.set_fix_size(size)
      Sketchup.write_default(PLUGIN_NAME, 'fix_width', size[0])
      Sketchup.write_default(PLUGIN_NAME, 'fix_height', size[1])
    end

    def self.add_action_callback
      @dialog.add_action_callback('ready') do |_action, width, height, _window|
        # load_su_dynamicomponents
        # p 'ready'

        @fix_width, @fix_height = get_fix_size
        if !@fix_width
          if IS_WIN
            w = if width > PLUGIN.settings[:dialog_options_min_w] + 15 + 50
                  width
                else
                  PLUGIN.settings[:dialog_options_min_w] + 50
                end
  
            h = if height > PLUGIN.settings[:dialog_options_min_h] + 15 + 50
                  height
                else
                  PLUGIN.settings[:dialog_options_min_h] + 50
                end
          else
            w = width
            h = height
          end
  
          size = [w, h]
          @on_check_dialog_size = true
          @dialog.set_size(size[0], size[1])
  
          # p "check size: #{[width, height]}, #{size}"
          @dialog.execute_script("checkSize(#{size})")
        end

        # Remove local store collection
        unless @clear_store_collection
          @dialog.execute_script("clearStoreCollection()")
          @clear_store_collection = true
        end

        refresh_dialog()
      end

      @dialog.add_action_callback('fix_size') do |_ac, width, height, old_size|
        p "get fix size: #{width}-#{height} (#{old_size})"

        @fix_height = old_size.last - height
        @fix_width = old_size.first - width
        set_fix_size([@fix_width, @fix_height])

        p [@fix_height, @fix_width]

        @dialog.set_size(width + 2 * @fix_width, height + 2 * @fix_height)
        @on_check_dialog_size = false

        set_dialog_size
      end

      @dialog.add_action_callback('say') do |a, *args|
        # p "say: #{args.length}"
        # ap args
      end

      @dialog.add_action_callback('saveSettings') do |a, settings|
        next unless @entity
        p "saveSettings::#{settings}"

        $settings = settings 

        @model.start_operation('saveSettings:', true)
        @entity.definition.set_attribute(PLUGIN_NAME, 'Dialog Settings', settings.to_json)
        @model.commit_operation
      end


      @dialog.add_action_callback('toggleCollection') do |a, show, dialog_size|
        toggle_collection(show, dialog_size)
      end

      @dialog.add_action_callback('pickModel') do |a, entity, keys|
        p 'pickModel'
        # ap entity
        # ap keys
        Collection3WH.download(entity)
      end

      @dialog.add_action_callback('saveData') do |a, data, action|
        next unless @entity
        p "Save UI::#{action}"

        @model.start_operation('Save UI', true)
        save_data(data)
        @model.commit_operation
      end

      @dialog.add_action_callback('saveTree') do |a, data|
        p 'Save Tree UI'
        @model.start_operation('Save Tree UI', true)
        TreeUI.new(@entity.definition).save(data)
        @model.commit_operation
      end

      @dialog.add_action_callback('refresh_dialog') do
        refresh_dialog

        if @missing && !@missing.empty?
          p "@missing", @missing.map{|a| a['key']}
          @model.start_operation('Remove Missing Attribute', true)
          @missing.each { |a| CURIC::DCUI::CustomAttr.remove(@entity.definition, a)}
          @model.commit_operation
        end
      end

      @dialog.add_action_callback('mouseFocusChanged') do |_contex, attribute|
        mouse_focus_dialog(attribute)
      end

      @dialog.add_action_callback('activeGroupsChanged') do |_a, active_groups|
        @model.start_operation('activeGroupsChanged', true)
        save_active_groups(@entity.definition, active_groups)
        @model.commit_operation
      end

      @dialog.add_action_callback('resizeDialog') do |_action, w, h|
        # p ['onResizeDialog', [w, h], "on_check_dialog_size: #{@on_check_dialog_size}"]

        next unless @dialog.visible? && !@on_check_dialog_size

        unless @selection.empty?
          if w >= PLUGIN.settings[:dialog_options_min_w]
            PLUGIN.settings[:dialog_options_w] = w
          end
          if h >= PLUGIN.settings[:dialog_options_min_h]
            PLUGIN.settings[:dialog_options_h] = h
          end
        end
      end

      @dialog.add_action_callback('attributeChangedProp') do |_action, attribute|
        @model.start_operation("Change Attribute Properties", true)
        CustomAttr.save_attribute(@entity.definition, attribute)
        @model.commit_operation
      end

      @dialog.add_action_callback('change_attr_value') do |_action, attribute|
        # p 'change_attr_value'
        # p 'attribute', attribute
        attr_name = attribute['name']
        @cache_attrs[attr_name] = attribute
      end

      @dialog.add_action_callback('change_attr_displaytype') do |_action, attribute|
        @model.start_operation('change display type', true)
        if attribute['display_type'] == 'MATERIAL'
          # refresh material
          mats = materials_data([attribute])
          mats.each do |name, value|
            @dialog.execute_script("app.data.materials['#{name}'] = #{value.to_json};")
          end
        end
        @model.commit_operation
      end

      @dialog.add_action_callback('changeAttrSuggestion') do |_action, attribute|
        # p 'changeAttrSuggestion', attribute['suggestions']
        @model.start_operation('change display type', true)
        @entity.definition.set_attribute(DCP_ATTR_D, attribute['name'] + '_suggestions', attribute['suggestions'])
        @model.commit_operation
      end
      
      @dialog.add_action_callback('editTooltip') do |_action, attribute|
        TooltipEditor.edit(attribute)
      end

      @dialog.add_action_callback('openpanelImageFile') do |_a, size|
        get_image_file(size)
      end
    
      @dialog.add_action_callback('addOptionImage') do |attribute, option_value|
        # p 'addOptionImage'
        # ap attribute
        # p option_value
        
        path = UI.openpanel
        next unless path

        # check size
        image_rep = Sketchup::ImageRep.new(path)
        if image_rep.height > 256 || image_rep.height > 256
          UI.messagebox('Image size too large.  Please select a photo smaller than 256x256px!')
          next
        end

        data64 = File.open(path, 'rb') do |img|
          'data:image/png;base64,' + Base64.strict_encode64(img.read)
        end

        data = {
          id: SecureRandom.uuid,
          base64: data64,
          path: path
        }

        # ap data

        @dialog.execute_script("app.rb_value = #{data.to_json};")
      end
        
      @dialog.add_action_callback('insertImageURLToLayout') do |_a, url|
        next unless url

        require 'open-uri'

        begin
          img = open(url)
          data64 = 'data:image/png;base64,' + Base64.encode64(img.read)
          data = {
            id: SecureRandom.uuid,
            base64: data64,
            url: url
          }
        rescue StandardError => e
          p e
          data = nil
        end
        
        @dialog.execute_script("app.rb_value = #{data.to_json};") if data
      end

      # AttrGroups
      @dialog.add_action_callback('changedGroups') do |_action, attr_groups|
        # p 'changedGroups'
        # ap attr_groups
        @model.start_operation('changedGroups', true)
        AttrGroups.save_groups(@entity.definition, attr_groups)
        @model.commit_operation
      end

      @dialog.add_action_callback('removeAttrGroup') do |_action, group|
        group = JSON.parse(group.to_json, symbolize_names: true)
        @model.start_operation('removeAttrGroup', true)
        AttrGroups.remove(@entity.definition, group)
        @model.commit_operation
      end

      @dialog.add_action_callback('updateAttrGroup') do |_action, group|
        UI.start_timer(0.1, false){
          update_group(group)
        }
      end

      @dialog.add_action_callback('accept') do |_action, _data|
        p 'accept'
      end

      @dialog.add_action_callback('show_tree') do
        show_tree_dialog
      end

      @dialog.add_action_callback('reset') do
        @model.start_operation('clear_attributes', true)
        clear_attributes
        @model.commit_operation

        refresh_dialog
      end

      @dialog.add_action_callback('apply') do |_action, data, oninput|
        # p [oninput, @oninput]
        if !oninput && @oninput
          # End of onChange
          @oninput = false
          CURIC::DCUI.dialog_observer.dc_observer.disable = false

          @model.commit_operation
          next
        end
  
        if oninput && !@oninput
          # Start input
          @oninput = true
          CURIC::DCUI.dialog_observer.dc_observer.disable = true
          Sketchup.active_model.start_operation('redraw', true)
        end

        if !oninput && !@oninput
          # p 'not on change => Start operation'
          Sketchup.active_model.start_operation('redraw', true)
        end

        # if @oninput
        #   CURIC::DCUI.dialog_observer.dc_observer.disable = true
        #   Sketchup.active_model.start_operation('redraw', true)
        # else
        #   Sketchup.active_model.start_operation('redraw', true)
        # end

        redraw = apply
        Sketchup.active_model.active_view.invalidate

        if redraw
          reset_cache
          # p "After apply => refresh dialog"
          # ! refresh_dialog by observer
          # refresh_dialog unless @oninput

          @disable_ui = true
          $dc_observers.get_latest_class.refresh_dialogs('Curic DCUI')
          @disable_ui = false
        end

        if !oninput && !@oninput
          # p 'not on change => commit_operation'
          @model.commit_operation
        end
      end

      @dialog.add_action_callback('openURL') do |_action, url|
        # UI.openURL(url)
        d = UI::HtmlDialog.new
        d.set_url(url)
        d.set_size(1000, 800)
        d.center
        d.show
      end

      @dialog.add_action_callback('place_component') do |_action, hash|
        # p 'place_component', hash
        PLUGIN.place_component(hash['path'])
      end

      @dialog.add_action_callback('toggleMode') do |_a, show|
        if show && !Object.const_defined?('CURIC::Studio')
          @dialog.execute_script('app.needLicense()')
        end
      end
    end

    def self.get_image_file(size)
      path = UI.openpanel
      # p path
      return unless path

      extname = File.extname(path).downcase
      if extname  == '.svg'
        str = File.open(path, 'rb') { |img| img.read }

        data = {
          id: SecureRandom.uuid,
          extname: extname,
          path: path,
          src: str
        }
      else
        image_rep = Sketchup::ImageRep.new(path)
        # p [image_rep.width, image_rep.height]
        if size && image_rep.width > size[0] || image_rep.height > size[1]
          UI.messagebox("Image size too large. Please select a photo smaller than #{size.join('x')}!")
          return
        end
  
        data64 = File.open(path, 'rb') do |img|
          'data:image/png;base64,' + Base64.strict_encode64(img.read)
        end
  
        data = {
          id: SecureRandom.uuid,
          extname: extname,
          base64: data64,
          path: path,
          width: image_rep.width,
          height: image_rep.height
        }
      end

      @dialog.execute_script("app.rb_value = #{data.to_json};")
    end

    def self.save_data(data)
      return unless data
       
      begin
        license = CURIC::Studio.extension_license.licensed
      rescue StandardError => e
        license = false
      end
  
      unless license
        begin
          Sketchup.status_text = 'Checking license...'
          return CURIC::Studio::Licensing.extension_license { 
            save_data(data)
          }
        rescue StandardError => e
          Sketchup.status_text = 'Check Falid!'
          return e
        end
      end

      return unless license

      p "save data: #{license}"

      save_active_groups(@entity.definition, data['active_groups'])

      TreeUI.new(@entity.definition).save(data)
      CustomAttr.save_tree(@entity.definition, data)
    end
    def self.clear_attributes
      @entity.definition.attribute_dictionaries.delete(PLUGIN_NAME)
    end

    def self.update_group(group)
      group = JSON.parse(group.to_json, symbolize_names: true)
      @model.start_operation('updateAttrGroup', true)
      AttrGroups.edit(@entity.definition, group)
      @model.commit_operation
    end

    def self.save_active_groups(definition, active_groups)
      definition.set_attribute(PLUGIN, 'active_groups', active_groups)
    end

    def self.get_active_groups(definition)
      definition.get_attribute(PLUGIN, 'active_groups', [])
    end

    def self.changed_groups(attr_groups)
      p 'changed_groups'
      # ap attr_groups
      save_attr_groups(attr_groups)
    end

    def self.save_attr_groups(definition, attr_groups)
      path = [PLUGIN_NAME, 'Groups']
      groups = AttributeHelper.dictionary(definition, path, true)

      groups[attr_groups['key']] = attr_groups.to_json
    end

    # Custom Record
    module AttrGroups
      include AttributeHelper

      module_function

      def save_groups(definition, groups)
        groups = JSON.parse(groups.to_json, symbolize_names: true)
        groups.each { |group| update(definition, group) }
      end

      def groups(definition)
        path = [PLUGIN_NAME, 'Groups']
        ad_groups = AttributeHelper.dictionary(definition, path, false)

        groups = []

        ad_groups&.each do |_id, json|
          groups << JSON.parse(json, symbolize_names: true)
        end

        groups
      end

      def add(definition, group)
        p 'add AttrGroups'

        update(definition, group)
      end

      def edit(definition, group)
        p 'edit AttrGroups'
        update(definition, group)
      end

      def remove(definition, group)
        path = [PLUGIN_NAME, 'Groups']
        groups = AttributeHelper.dictionary(definition, path)
        groups.delete_key(group[:key])
      rescue StandardError => e
        p e
      end

      def update(definition, group)
        path = [PLUGIN_NAME, 'Groups']
        groups = AttributeHelper.dictionary(definition, path, true)

        groups[group[:key]] = group.to_json
      end
    end

    def self.selection_changed
      return unless @dialog && @dialog.visible?
      
      # p "selection_changed => refresh dialog"
      refresh_dialog

      set_dialog_size
    end

    def self.rename_attribute(entity, name, new_name, new_label)
      if @dialog && @dialog.visible?
        @dialog.execute_script("renameAttribute('#{name}', '#{new_name}', '#{new_label}')")
      end
      # rename in attributeDictionaries

      m = Sketchup.active_model
      # CustomAttr.remove()
      attribute_info = CustomAttr.get_data(entity.definition, name)
      # p 'rename_attribute'
      # ap attribute_info

      if attribute_info
        attribute_info['key'] = new_name
        attribute_info['name'] = new_name
        m.start_operation('Rename attribute', true, false, true)
        CustomAttr.update(entity.definition, attribute_info)
        CustomAttr.remove(entity.definition, name)

        # Change in tree
        CURIC::DCUI::TreeUI.rename_attribute(entity.definition, name, new_name)

        m.commit_operation
      end
    end

    def self.entity_redrawed(entity)
      return unless entity && !entity.deleted?
      return unless @dialog && @dialog.visible?
      return unless @selection.to_a.include?(entity)

      # p "entity_redrawed: #{entity.definition.name}"

      # check is on animation by interact tool
      tool = Sketchup.active_model.tools.active_tool
      if tool && tool.is_a?(DCInteractTool)
        is_animating = tool.instance_variable_get(:@is_animating)
        p "is_animating: #{is_animating}"
        return if is_animating
      end

      refresh_dialog
    end

    def self.set_dialog_size
      return unless @dialog_settings

      if @dialog_settings['fixed_width'] && @dialog_settings['fixed_height']
        width = @dialog_settings['width']
        height = @dialog_settings['height']
        width += 340 if @show_collection
        @dialog.set_size(width, height)
      end
    end

    # Khi User nhấn Apply trên dialog
    # Xác nhận các thay đổi attrs và thực hiện redraw components
    def self.apply
      # Kiểm tra các thay đổi của các attr
      # Attr có thay đổi value thì set value
      # Nếu có một attr thay đổi value thì redraw DC

      # Kiểm tra trong @cache_attrs và dữ liệu push lên dialog trong lần mới nhất
      # p "apply"
      # p "@data_attrs: #{@data_attrs.length}"
      # p "@data_products: #{@data_products.length}"
      # ap @data_attrs

      return if @cache_attrs.empty?
      return if @data_attrs.empty?
      return unless @data_attrs['attributeDictionaries']
      return unless @data_attrs['attributeDictionaries']['dynamic_attributes']
      if @data_attrs['attributeDictionaries']['dynamic_attributes'].empty?
        return
      end
      dc = @data_attrs['attributeDictionaries']['dynamic_attributes']

      # @model.start_operation('apply', true)
      redraw_dc = false

      need_redraws = []
      @dc = $dc_observers.get_latest_class

      @cache_attrs.each do |key, data|
        @data_products.each do |e, data_attrs|
          dc = data_attrs['attributeDictionaries']['dynamic_attributes']
          next unless dc[key]

          new_value = data['value']
          old_value = dc[key]['value']
          changed = (new_value != old_value)
          next unless changed

          p "#{key} => changed: #{old_value} => #{new_value}"
          
          @dc.set_attribute(e, key, new_value)

          formula = @dc.get_attribute_formula(e, key)
          @dc.set_forced_config_value(e, key, new_value) if formula.nil?
          
          if @dc.respond_to?(:store_nominal_size) && %w[lenx leny lenz].include?(key)
            lenx = @dc.get_attribute_value(e, 'lenx')
            leny = @dc.get_attribute_value(e, 'leny')
            lenz = @dc.get_attribute_value(e, 'lenz')

            lens = {
              'lenx' => lenx,
              'leny' => leny,
              'lenz' => lenz
            }
            lens[key] = new_value

            # @dc.store_nominal_size(e, lens['lenx'], lens['leny'], lens['lenz'])
          end

          need_redraws << e unless need_redraws.include?(e)

          redraw_dc = true
        end
      end

      if redraw_dc
        DCProgressBar.clear
        dc = $dc_observers.get_latest_class
        need_redraws.each do |e| 
          begin
            dc.redraw(e, false) 
          end
        end
      end

      # @model.commit_operation

      redraw_dc
    end

    def self.set_attribute_value(entity, key, value)
      p 'set_attribute_value', key, value
      # ! check instance attributes
      # if @entity.get_attribute('dynamic_attributes', key) || @entity.is_a?(Sketchup::Group) ||
      #   @entity.is_a?(Sketchup::ComponentDefinition) || @entity.is_a?(Sketchup::Model)
      #   metadata_entity = @entity
      # else
      #   metadata_entity = @entity.definition
      # end

      entity.definition.set_attribute('dynamic_attributes', key, value)
      entity.set_attribute('dynamic_attributes', key, value)
    end

    # refresh
    def self.refresh_dialog(r_w = true)
      return if @disable_ui
      return unless @dialog && @dialog.visible?

      # p 'refresh dialog'
      
      refresh_workspace if r_w

      # Xóa dữ liệu cache các attribute
      reset_cache

      dcs = @selection.find_all { |e| PLUGIN.dc?(e) }

      objects = dcs
      objects.uniq!

      materials = []

      @entity = nil
      @dialog_settings = nil

      @dialog.execute_script("updateSU_info(#{su_info().to_json})")

      if !objects.empty?
        refresh_component(objects)
      elsif !materials.empty?
        refresh_material(materials)
      else
        # clear data
        clear_data
      end

      return unless @entity

      @dialog_settings = dialog_setting(@entity.definition)
      data = { settings: @dialog_settings }
      @dialog.execute_script("app.dialog_settings = #{data.to_json}.settings;")
    end

    def self.su_info()

      # json = "{"
      # json += "'version': '" + Sketchup.version.to_s + "'"
      # json += ", 'language': '" + Sketchup.get_locale.to_s + "'"
      # json += ", 'is_pro': " + Sketchup.is_pro?.to_s 
      # json += ", 'dc_version': '" + $dc_extension.version.to_s + "'"
      # json += ", 'units': '" + get_default_units + "'"
      # json += ", 'decimal_delimiter': '" + get_decimal_delimiter + "'"
      # json += ", 'temp_path': '" + File.expand_path(get_temp_path) + "'"
      # json += ", 'simplify': " + @simplify.to_s
  
      # dc_help_url = Sketchup.get_datfile_info 'DC_HELP', 
      #   'http://help.sketchup.com/article/115543'
  
      # dc_functions_url = Sketchup.get_datfile_info 'DC_FUNCTIONS', 
      #   'http://help.sketchup.com/article/108144'
  
      # dc_online_url = Sketchup.get_datfile_info 'GENERATE_REPORT',
      #   'https://3dwarehouse.sketchup.com/generatereport.html'
  
      # json += ", 'dc_help_url': '" + dc_help_url + "'"
      # json += ", 'dc_functions_url': '" + dc_functions_url + "'"
      # json += ", 'dc_online_url': '" + dc_online_url + "'"
  
      # # Add information regarding default formatting.
      # options = Sketchup.active_model.options['UnitsOptions']
      # json += ", 'length_format': '" + get_length_format + "'"
      # json += ", 'length_unit': '" + get_length_unit + "'"
      # json += ", 'length_precision': " + options['LengthPrecision'].to_s
      # json += ", 'angle_precision': " + options['AnglePrecision'].to_s
  
      # json += ", 'strings': " + "{"
      # $dc_strings.GetStrings().keys.each do |string|
      #   json += '"' + clean_for_json(string) + '": "' + 
      #     clean_for_json($dc_strings.GetString(string)) + '",'
      # end

      # json = json.chop + "}}";

      info = {}
      info['language'] = Sketchup.get_locale.to_s
      info['units'] = @dc.get_default_units
      info['decimal_delimiter'] = @dc.get_decimal_delimiter
      info['length_format'] = @dc.get_length_format
      info['length_unit'] = @dc.get_length_unit

      options = Sketchup.active_model.options['UnitsOptions']
      info['length_precision'] = options['LengthPrecision']
      info['angle_precision'] = options['AnglePrecision']

      info
    end

    def self.dialog_setting(definition)
      default = {
        "fixed_height" => false,
        "fixed_width" => false,
        "width" => 350,
        "height" => 500,
        "min_width" => 350,
        "max_width" => 1200,
        "min_height" => 350,
        "max_height" => 1200,
        "tooltip_hide_after" => 0,
        "tooltip_open_delay" => 0.5,
        "use_max_height" => false,
        "use_max_width" => false,
        "use_min_height" => false,
        "use_min_width" => false,
        "password" => ""
      }

      settings = definition.get_attribute(PLUGIN_NAME, 'Dialog Settings', '{}')
      settings = JSON.parse(settings)
      settings = default.merge(settings)

      settings
    end

    def self.refresh_component(products)
      data_products = products.each_with_object({}) { |e, h| h[e] = component_attributes(e) }
      # ap data_products

      @entity = products[0]
      data = (data_products.values[0] if @entity)

      if products.length > 1
        # find same attribute: chỉ tìm các attribute hiển thị trên dialog
        match_attrs = matching_options(data_products.values)
      else
        match_attrs = nil
      end

      if data
        data['selected_component'] = true
        data['multi_object'] = !match_attrs.nil?
        data['products'] = products
        data['matching_options'] = match_attrs

        # Set same options match value
        if match_attrs && !match_attrs.empty?
          match_attrs.each do |name, d|
            next unless d['access'] == 'LIST'

            data['attributeDictionaries']['dynamic_attributes'][name]['options'] = d['options']
          end
        end

        # group attrbutes
        # attr_groups = []
        attr_groups = get_attribute_groups(@entity.definition, data)

        # attr_groups << {
        #   key: 'Group A',
        #   prefix: nil,
        #   attributes: [],
        #   label: 'Group A'
        # }
        
        data['attr_groups'] = attr_groups

        # materials
        material_attrs = TreeUI.flat_dc_attributes(attr_groups).find_all { |a|
          a['access'] == 'LIST' &&
          (a['name'] == 'material' ||
          a['display_type'] == 'MATERIAL')
        }
        data['materials'] = materials_data(material_attrs)
        

        data['model_id'] = @entity.definition.get_attribute('3DWarehouse', 'ModelID')
        data['active_groups'] = get_active_groups(@entity.definition)
      end

      @dialog.execute_script("updateData(#{data.to_json})")
      
      # data = JSON.parse(data.to_json, symbolize_names: true)
      # @dialog.execute_script("updateData(#{JSON.pretty_generate(data)})")

      @data_attrs = data
      @data_products = data_products
    end

    def self.get_attribute_groups(definition, data)
      attr_groups = AttrGroups.groups(definition)

      attrs = data['attributeDictionaries']['dynamic_attributes'].keys.find_all do |name|
        access = data['attributeDictionaries']['dynamic_attributes'][name]['access']
        access && access != 'NONE'
      end

      display_attrs = data['attributeDictionaries']['dynamic_attributes'].keys.find_all do |name|
        access = data['attributeDictionaries']['dynamic_attributes'][name]['access']
        access && access != 'NONE'
      end

      dynamic_attributes = data['attributeDictionaries']['dynamic_attributes'].clone

      # set attribute info of attr_groups from dynamic_attributes

      tree_ui = TreeUI.new(definition).to_h
      if tree_ui
        # Tree chỉ lưu id-key của các attribute
        # Set dynamic_attributes và CustomAttr data
        @missing = []
        set_attributes_data(definition, tree_ui, dynamic_attributes)
      else
        tree_ui = []
      end

      # Đối chiếu attribute: đã phân loại, đổi tên...
      tree_attributes = TreeUI.flat_dc_attributes(tree_ui)

      # p 'tree_attributes'
      # ap tree_attributes

      # Các attribute chưa được phân nhóm
      unclassified = dynamic_attributes.find_all do |name, _info|
        name != 'name' &&
          display_attrs.include?(name) &&
          !tree_attributes.find { |a| a['name'] == name }
      end

      unclassified = unclassified.to_h

      # puts "unclassified: #{unclassified.keys}" unless unclassified.empty?

      unless unclassified.empty?
        # Push to Default Group
        defaul_group = tree_ui.find { |g| g['label'] == 'Attributes' || g[:label] == 'Attributes' }

        unless defaul_group
          tree_ui << defaul_group = {
            'key' => 'Attributes',
            'label' => 'Attributes',
            'access' => 'Group',
            'prefix' => nil,
            'attributes' => []
          }
        end

        unclassified.each do |_name, a|
          defaul_group['attributes'] << a
        end
      end

      tree_ui
    end

    def self.set_attributes_data(definition, attributes, dc_attrs)
      # check missing
      missing = []
      attributes.each do |attribute|
        key = attribute['key']
        dc_data = dc_attrs[key]
        if dc_data
          # attribute.merge!(dc_data) {|key, oldVal, newVal| newVal }
        elsif !TreeUI.custom_attr?(attribute)
          p "Missing attribute: #{attribute['key']}:#{attribute['formlabel']}"
          # ap attribute

          missing << attribute
          attributes.delete(attribute)
          next
        end

        custom = CustomAttr.get_data(definition, key)
        attribute.merge!(custom) { |key, oldVal, newVal| newVal }
        attribute.merge!(dc_data) if dc_data

        if !TreeUI.custom_attr?(attribute)
          attribute['display_type'] = 'DEFAULT' if !attribute['display_type']
        end

        attribute['on_edit'] = false
        attribute.delete('onEdited')
        attribute.delete('path')
        
        attribute['label_style'] = [] unless attribute['label_style']

        if attribute['attributes']
          set_attributes_data(definition, attribute['attributes'], dc_attrs)
        end

        if attribute['legends']
          set_attributes_data(definition, attribute['legends'], dc_attrs)
        end
      end

      # missing.each { |a| attributes.delete(a) }

      @missing += missing
    end

    def self.set_attribute_data(definition, attribute, dc_attrs)
      key = attribute['key']
      dc_data = dc_attrs[key]
      if dc_data
        # attribute.merge!(dc_data) {|key, oldVal, newVal| newVal }
      elsif !TreeUI.custom_attr?(attribute)
        # remove form tree
        p "Missing attribute: #{attribute['key']}:#{attribute['formlabel']}"
        # ap attribute
        # Remove attribute
        CURIC::DCUI::CustomAttr.remove()
      end

      custom = CustomAttr.get_data(definition, key)
      attribute.merge!(custom) {|key, oldVal, newVal| newVal }

      if !TreeUI.custom_attr?(attribute)
        attribute['display_type'] = 'DEFAULT' if !attribute['display_type']
      end

      attribute.delete('onEdited')
      attribute.delete('path')

      if attribute['attributes']
        attribute['attributes'].each { |a|
          # check missing

          set_attribute_data(definition, a, dc_attrs) 
        }
      end

      if attribute['legends']
        attribute['legends'].each {|a| set_attribute_data(definition, a, dc_attrs)}
      end

      attribute['on_edit'] = false
    end

    def self.refresh_material(materials)
      p 'refresh_material', materials

      material = materials[0]

      itemcode = PLUGIN.material_itemcode(material)
      itemcodes = Material.codes(material)

      data = {}
      data['selected_material'] = true

      # Write thumbnail
      name = material.display_name
      thumbnail = File.join(Sketchup.temp_dir, "#{name}.jpg")
      status = material.write_thumbnail(File.join(Sketchup.temp_dir, "#{name}.jpg"), 256)
      p "write_thumbnail: #{status}"

      material_data = {
        id: material.object_id,
        itemcode: (itemcode || ''),
        codes: itemcodes,
        thumbnail: thumbnail,
        name: name
      }

      data['material'] = material_data
      @dialog.execute_script("updateData(#{data.to_json})")
    end

    def self.clear_data
      data = nil
      data_products = []
      odoo_datas = []

      @dialog.execute_script("updateData(#{data.to_json})")
      @dialog.execute_script("app.odoo_datas = #{odoo_datas.to_json};")

      @data_attrs = data
      @data_products = data_products
    end

    # Get Dynamic Component Attributes
    # "lenx" => {
    #               "value" => "39.37007874015748",
    #               "label" => "LenX",
    #             "options" => "&",
    #             "access" => "TEXTBOX",
    #           "formlabel" => "LenX",
    #               "units" => "MILLIMETERS",
    #       "formulaunits" => "INCHES"
    #   }
    # 
    # @param <Sketchup::ComponentInstance> entity
    # 
    # @return <Hash>
    def self.component_attributes(entity)
      json = CURIC::DC.entity_in_tree(entity)
      data = JSON.parse(json)
      # p 'entity_in_tree'
      ad = data['attributeDictionaries']
      # if data['attributeDictionaries']
      #   data['attrs'] = data.delete('attributeDictionaries')
      #   if data['attrs']['dynamic_attributes']
      #     data['attrs']['dc'] = data['attrs'].delete('dynamic_attributes')
      #   end
      # end
      return unless ad && ad['dynamic_attributes']

      dc = ad['dynamic_attributes']

      # format data, add new function...
      format_dc_attribute(entity, dc)

      # thumbnail
      name = entity.definition.name
      name.tr!('#', '_')
      thumbnail = File.join(Sketchup.temp_dir, "#{name}.png")

      begin
        FileUtils.remove(thumbnail)
      rescue => exception
        # 
      end

      entity.definition.refresh_thumbnail
      entity.definition.save_thumbnail(thumbnail)

      data['thumbnail'] = thumbnail

      # subproducts
      entities = entity.definition.entities
      
      data
    end

    def self.format_dc_attribute(entity, dc)
      dc.each do |name, value|
        # Add Name to value
        value['name'] = name
        value['key'] = name

        value['on_edit'] = false
        value['render'] = true

        # next 
        value['value'] = unenscape_html(value['value'])

        # "&A=0.39370078740157477&b=3.937007874015748&"
        formlabel = value['formlabel']
        if value['formlabel']
          value['formlabel'] = unenscape_html(value['formlabel'])
        end
        
        # Format options
        if value['access'] == 'LIST'
          options = value['options']
          options.gsub!('&#92;', '\\')
          options.gsub!('&#47;', '/')

          array = options.split('&')
          array.delete('')

          ops = {}
          array.each do |op|
            k, v = op.split('=')
            f_k = unenscape_html(k)
            ops[f_k] = v
          end

          value['options'] = ops
        end

        # ! Nếu value của option là length sẽ có sai số => check, làm tròn

        if value['access'] == 'TEXTBOX' || value['access'] == 'VIEW'
          # value['display_value'] = format_units_value(value['value'], value['units'], dc['_lengthunits']['value'])
        end
      end
    end

    def self.unenscape_html(value)
      value.gsub!('&#92;', '\\')
      value.gsub!('&#47;', '/')
      return value
      return CGI.unescape(value)
    end

    def self.matching_options(datas)
      def self.same_options(v1, v2)
        return v1['options'] == v2['options'] if v1['access'] == 'LIST'

        true
      end

      attrbutes = {}
      datas[0]['attributeDictionaries']['dynamic_attributes'].each do |k, v|
        next if !v['access'] || v['access'] == 'NONE'

        # tất cả các component còn lại có tồn tại k và cùng kiểu hiển thị
        same = datas[1..-1].all? do |d|
          dc_at = d['attributeDictionaries']['dynamic_attributes']
          dc_at.key?(k) && 
            dc_at[k]['access'] == v['access'] &&
            dc_at[k]['formlabel'] == v['formlabel'] # &&
          # same_options(v, dc_at[k])
        end

        next unless same

        same_listview = false
        list = {}
        if v['access'] == 'LIST'
          # lấy phần option chung
          list = v['options'].select do |label, value|
            datas[1..-1].all? do |d|
              dc_at = d['attributeDictionaries']['dynamic_attributes']
              dc_at[k]['options'].key?(label) && dc_at[k]['options'][label] == value
            end
          end

          next if list.empty?

          same_listview = true
        end 

        if same_listview && !list.empty?
          new_value = v.clone
          new_value['options'] = list
          # Set new value

          v = new_value
        end

        attrbutes[k] = v
      end
      
      attrbutes
    end

    def self.toggle_collection(show, dialog_size)
      p 'toggle_collection', [show, dialog_size, [@fix_width, @fix_height]]
      
      @show_collection = show
      width, height = dialog_size
      if(show)
        new_width = width + 340
      else
        new_width = width - 340
      end

      @animation = ResizeDialog.new(@dialog)
      @animation.width = width
      @animation.height = height + @fix_height
      @animation.new_width = new_width + @fix_width
      @animation.onDone {
        UI.start_timer(0.5, false){ @dialog.execute_script("app.layoutDialog()") }
      }

      Sketchup.active_model.active_view.animation = @animation
    end

    def self.show_tree_dialog
      d = IS_WIN ? 15 : 0
      @tree_dialog = UI::HtmlDialog.new(
        dialog_title: 'Tree View',
        preferences_key: "#{PLUGIN_NAME}.dialog.define_tree",
        scrollable: true,
        resizable: true,
        width: 290,
        height: 500,
        left: 100,
        top: 100,
        min_width: 265 + d,
        min_height: 330 + d,
        style: UI::HtmlDialog::STYLE_UTILITY
      )

      html = File.join(@path, 'html/attr_groups.html')
      @tree_dialog.set_file(html)

      @tree_dialog.add_action_callback('ready') do |_action|
        p 'ready'
        # parse group json data
        data = []
        @data_attrs['attr_groups'].each do |h|
          children = h[:attrs].map do |a|
            {
              label: @data_attrs['attributeDictionaries']['dynamic_attributes'][a]['label'],
              name: a,
              is_left: true,
              is_group: false
            }
          end
          node = {
            label: h[:label],
            children: children,
            is_left: false,
            is_group: true
          }
          data << node
        end

        @tree_dialog.execute_script("updateData(#{data.to_json})")
      end

      @tree_dialog.show
    end

    def self.reset_cache
      @cache_attrs = {}
    end

    def self.default_dc_attributes
      dc = $dc_observers.get_latest_class
      conv = dc.instance_variable_get(:@conv)
      reserved_attribute_group = conv.instance_variable_get(:@reserved_attribute_group)
      reserved_attribute_group.keys
    end

    def self.materials_data(attrs=nil)
      data = {}

      if attrs
        names = attrs.map { |a| a['options'].values }.flatten.uniq
        names.map! { |name| unenscape_html(name) }
        # ap names
        materials = []
        names.each{ |name|
          m = @materials[name] || @materials["[#{name}]"]
          if m
            materials << m
            next
          end
            
          m = name[0] == '#' ? @materials[name[1..-1]] : nil
          materials << m if m
          next if m

          begin
            color = Sketchup::Color.new(name)
            m = @materials.add(name)
            m.color = color
            materials << m
          rescue => exception
            p exception, name
            m = nil
          end
        }
      else
        materials = @materials
      end

      materials.each do |m|
        name = m.name == m.display_name ? m.name : m.display_name
        data[name] = {
          key: name,
          label: name,
          value: name,
          thumbnail: material_thumbnail(m),
          color: "rgba(#{m.color.red}, #{m.color.green}, #{m.color.blue}, #{m.color.alpha})"
        }
      end

      # Materials from library

      data
    end

    def self.material_thumbnail(material, resolution = 64)
      name = material.name == material.display_name ? material.name : material.display_name
      name.tr!('#', '_')
      thumbnail = File.join(Sketchup.temp_dir, "#{name}.png")

      # p "material_thumbnail: #{material.name}"
      begin
        FileUtils.remove(thumbnail)
      rescue => exception
        # 
      end

      texture = material.texture
      if texture
        if texture.image_height > texture.image_width
          resolution = resolution > texture.image_width ? texture.image_width - 1 : resolution
        else
          resolution = resolution > texture.image_height ? texture.image_height - 1 : resolution
        end
      end

      status = material.write_thumbnail(thumbnail, resolution)
      # p "material.write_thumbnail: #{name} => #{status}"
      thumbnail
    end

    # value_in_string : current value in string
    # units : display units on options dialog
    # lengthunits : entity units of dynamic component

    # @return String
    def self.format_units_value(value_in_string, units, _lengthunits)
      length = value_in_string.to_f.to_l
      case units
      when 'DEFAULT'
        case Sketchup.active_model.options['UnitsOptions']['LengthUnit']
        when 0
          length.to_f.to_s
        when 1
          length.to_feet.to_s
        when 2
          "#{length.to_mm} mm"
        when 3
          "#{length.to_cm} cm"
        when 4
          "#{length.to_m} m"
          end

        "#{length.to_mm} mm"
      when 'MILLIMETERS'
        "#{length.to_mm.round(1)} mm"
      when 'CENTIMETERS'
        "#{length.to_cm.round(2)} cm"
      when 'METERS'
        "#{length.to_m.round(3)} m"
      when 'INCHES'
        length.to_f.round(3).to_s
      when 'FEET'
        length.to_feet.round(3).to_s
      when 'STRING'
        value_in_string
      when 'DEGREES'
        "#{value_in_string}°"
      when 'DOLLARS'
        "$#{value_in_string.to_f.round(2)}"
      end
    end

    # Sử dụng các js function của Dynamic Component
    def self.load_su_dynamicomponents
      js_files = Sketchup.find_support_files('js', 'Plugins/su_dynamiccomponents/js')
      files = [
        'json2.min.js',
        'dcbridge.js',
        'converter.js',
        'components.js',
        'manager.js',
        'configurator.js',
        'functions.js'
      ]

      hash = files.each_with_object({}) do |file, h|
        h[file] = js_files.find { |path| File.basename(path) == file }
      end
      # hash.values.each do |path|
      @dialog.execute_script("dynamicallyLoadScript(#{hash.values})")
      # end
    end

    def self.set_skp_info
      su.info['units']
    end
  end
end
