module CURIC::DCUI
  # Nhóm các option
  # Material with thumbnail

  module TooltipEditor

    extend UIMethods

    class << self
      attr_reader :dialog
    end
    @disable_ui = false

    def self.edit(attribute)
      # p "edit tooltip"
      # ap attribute
      @attribute = attribute

      if @dialog && @dialog.visible?
        @dialog.bring_to_front
        refresh_dialog
      else
        toggle_dialog
      end
    end

    def self.close_dialog
      toggle_dialog if @dialog && @dialog.visible?
    end

    def self.toggle_dialog
      return @dialog.close if @dialog && @dialog.visible?

      refresh_workspace
      @dialog = UI::HtmlDialog.new(
        dialog_title: 'Tooltip Editor',
        preferences_key: "#{PLUGIN_NAME}.tooltip.dialog.options",
        scrollable: true,
        resizable: true,
        width: 500,
        height: 500,
        left: 100,
        top: 100,
        min_width: 400,
        min_height: 600,
        max_width: 1000,
        max_height: 1000,
        style: UI::HtmlDialog::STYLE_UTILITY
      )

      html = File.join(@path, 'html/tooltip.html')
      @dialog.set_file(html)

      add_action_callback

      # remove observer when dialog on closed
      @dialog.set_on_closed { PLUGIN.detach_dialog_observers }

      @dialog.show
    end

    def self.add_action_callback
      @dialog.add_action_callback('ready') do |_action, width, height, _window|
        refresh_dialog
      end

      @dialog.add_action_callback('refresh_dialog') do
        refresh_dialog
      end

      @dialog.add_action_callback('apply') do |_action, attribute|
        apply(attribute)
      end

      @dialog.add_action_callback('remove') do |_action, attribute|
        attribute['tooltip'] = nil;
        apply(attribute)
      end

      @dialog.add_action_callback('cancel') do
        @dialog.close
      end
    end
  
    def self.apply(attribute)
      OptionsUI.dialog.execute_script("updateTooltip(#{attribute.to_json})")
      @dialog.close
    end

    def self.refresh_dialog
      if !@attribute['tooltip']
        @attribute['tooltip'] = {}
        @attribute['tooltip']['width'] = 150
        @attribute['tooltip']['height'] = 200
        @attribute['tooltip']['code'] = ''
      end

      @dialog.execute_script("editAttribute(#{@attribute.to_json})")
    end
  end
end
