module CURIC::DCUI
  class TreeUI
    def self.build_tree(data)
      data['attr_groups'].map { |a| build_attr_data(a) }
    end

    def self.build_attr_data(a)
      d = {}
      d['key'] = a['key']
      d['access'] = a['access']
      d['formlabel'] = a['formlabel'] if a['formlabel']

      if a['access'] == 'Fieldset'
        d['legends'] = a['legends'].map { |a| build_attr_data(a) }
      end

      if a['attributes']
        d['attributes'] = a['attributes'].map { |a| build_attr_data(a) }
      end

      d
    end

    def self.flat_dc_attributes(tree, attrs = [])
      return attrs unless tree
      
      tree.each do |attribute|
        attrs << attribute unless custom_attr?(attribute)

        if attribute['attributes']
          self.flat_dc_attributes(attribute['attributes'], attrs) 
        end 
        
        if attribute['legends']
          self.flat_dc_attributes(attribute['legends'], attrs) 
        end
      end

      return attrs
    end

    def self.rename_attribute(definition, old_name, new_name, attributes = nil)
      unless attributes
        data = definition.get_attribute(PLUGIN_NAME, 'tree_ui', 'null')
        @rename = false
        tree = JSON.parse(data)
        attributes = tree
      end

      return unless attributes

      attributes.each do |attribute|
        if attribute['key'] == old_name
          @rename = true
          attribute['key'] = new_name
          attribute['name'] = new_name
        end

        if attribute['attributes']
          rename_attribute(definition, old_name, new_name, attribute['attributes'])
        end 
        
        if attribute['legends']
          rename_attribute(definition, old_name, new_name, attribute['legends'])
        end
      end

      if tree
        # p 'rename_attribute on tree'
        # p "@rename: #{@rename}"
        # ap tree
        definition.set_attribute(PLUGIN_NAME, 'tree_ui', tree.to_json)
        @rename = false
      end
    end

    def self.custom_attr?(attribute)
      # p'custom_attr'
      # ap attribute
      access = attribute['access'] || attribute[:access]
      %w[Group Block Fieldset Column Legend Image FloatInput Section Divider].include?(access)
    end

    def initialize(definition)
      @definition = definition
    end

    def to_h
      json = @definition.get_attribute(PLUGIN_NAME, 'tree_ui', 'null')
      tree = JSON.parse(json)
      TreeUI.remove_duplicate(tree) if duplicate?(tree)

      tree
    end

    def save(data)
      p "save tree"
      tree = TreeUI.build_tree(data)
      TreeUI.remove_duplicate(tree) if duplicate?(tree)
     
      @definition.set_attribute(PLUGIN_NAME, 'tree_ui', tree.to_json)
    end

    def duplicate?(tree)
      flat = TreeUI.flat_dc_attributes(tree)
      duplicate = flat.find_all{ |a| flat.count(a) > 1 }.uniq

      !duplicate.empty?
    end

    def self.remove_duplicate(tree, duplicate = nil, remove=[])
      if duplicate == nil
        flat = TreeUI.flat_dc_attributes(tree)
        duplicate = flat.find_all{ |a| flat.count(a) > 1 }.uniq

        p ['duplicate', duplicate.map { |a| a['key'] }]

        duplicate = duplicate.each_with_object({}) { |a, h| h[a] = 0 }
      end
      return unless duplicate && !duplicate.empty?

      tree.each_with_index do |attribute, index|
        dup = duplicate.keys.find { |a| !custom_attr?(attribute) && a['key'] == attribute['key'] }

        if dup
          duplicate[dup] += 1

          if duplicate[dup] > 1
            tree.delete_at(index)
            remove << attribute
          end
        end

        if attribute['attributes']
          self.remove_duplicate(attribute['attributes'], duplicate, remove) 
        end 
        
        if attribute['legends']
          self.remove_duplicate(attribute['legends'], duplicate, remove) 
        end
      end

      return remove
    end
  end
end
