

// Attribute
Vue.component("attribute", {
  model: {
    prop: "attribute",
    event: "change",
    event: "input",
    event: "submit",
    event: "select",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    },
    group: {
      type: Object,
    },
    layout: {
      type: Object,
    },
    index: {
      type: Number,
    },
    disabled: {
      default: false,
      type: Boolean,
    }, 
    reverse: {
      default: false,
      type: Boolean,
    }, 
    onEdit: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
  },
  template: `
  <table :class="attributeClass">
    <tr class="attribute-input">
      <template v-if="attribute.access == 'Block'">
        <td>
          <block-attribute v-model="attribute"
          :ref="attribute.path"
          :parent="parent"
          :on-edit="onEdit"
          :disabled="disabled"
          :display_action="display_action">
          </block-attribute>
        </td>
      </template>
      <template v-else>
        <!-- handle -->
        <td>
          <handle-attribute v-if="onEdit && display_action"></handle-attribute>
        </td>

        <!-- Icon -->
        <td class="minimum-width">
          <div v-if="attribute.image && attribute.access != 'Image'" class="input-icon">
            <img :src="attribute.image ? attribute.image.base64 : ''" alt="icon">

            <!-- <el-image
              :src="attribute.image ? attribute.image.base64 : ''"
              atl="icon" fit="scale-down">

              <div slot="error"><i class="fa fa-image fa-2x"></i></div>
            </el-image> -->
          </div>
        </td>

        <!-- Item -->
        <td>
          <template v-if="$root.isCustomAttribute(attribute)">
            <template v-if="attribute.access == 'Image'">
              <image-attribute v-model="attribute"
              :parent="parent"
              :disabled="disabled"
              :on-edit="onEdit"
              :display_action="display_action"
              ></image-attribute>
            </template>
            <template v-else>
              <custom-attribute v-model="attribute"
                :parent="parent"
                :disabled="disabled"
                :on-edit="onEdit"
                :display_action="display_action"
                ></custom-attribute>
            </template>
          </template>

          <template v-else>
            <el-form-item :label-width="attribute.hide_label ? '0px' : ''">
              <!-- Label -->
              <template slot="label" v-if="(reverse == false || attribute.display_type == 'SWITCH')">
                <attribute-label v-model="attribute" :on-edit="onEdit" :path="attribute.path"></attribute-label>
              </template>
              
              <attribute-input
                v-model="attribute"
                @change="$emit('change', attribute)"
                @input="$emit('input', attribute)"
                @submit="$emit('submit')"
                @mouseover="$root.attributeMouseover(attribute)"
                @mouseleave="$root.attributeMouseleave(attribute)"
                v-bind:disabled="disabled"
                v-bind:reverse="reverse"
                v-bind:value="attribute.value"
                v-bind:display_value="attribute.display_value"
                v-bind:on-edit="onEdit"
              >
              </attribute-input>
            </el-form-item>
          </template>
        </td>

        <td class="action-right">
          <edit-attribute  v-if="display_action"
            v-model="attribute" :parent="parent" :on-edit="onEdit">
          </edit-attribute>
        </td>
      </template>
    </tr>
  </table>
  `,
  methods: {
    selectAttribute(){
      // remove all selected attributes
      clearSelectedAttribute();
      this.attribute.selected = true;

      // console.log(this.$parent.$el.children)
      // children = this.$parent.$el.children;
      // for (let i = 0; i < children.length; i++) {
      //   const c = children[i];
      //   c.className = 'attribute-item'
      // }
      // console.log(this.$el.parentNode.className)
      // this.$el.parentNode.className = 'attribute-item selected'

      this.$emit('select', this.attribute);
    }
  },
  watch: {
    // disabled(n, o){
    //   console.log(this.attribute.access, 'change disable', o, '=>', n)
    // }
  },
  computed: {
    attributeClass(){
      var c = 'table-layout-attributes ' + this.attribute.access;
      if(this.onEdit) {c += ' on-edit'};
      if(this.attribute.on_edit) {c += ' attribute-on-edit'};
      c += ' attribute-is-' + (this.attribute.hide ? 'hide' : 'show')
      return c; 
    }
  },
  mounted: function (){
    // console.log('attribute mounted', this.attribute.formlabel);
    var self = this;
    this.attribute.onEdited = function(action) {
      console.log('attribute.onEdited', action)
      self.$root.attributeChangedProp(self.attribute)
    }
  },
  created: function(){
    // console.log('attribute create', this.attribute.formlabel)
    if (!this.attribute.on_edit){
      // this.attribute.parent = this.parent;
      
      if(!this.parent.path){
        this.parent.path = ''
      }
  
      this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`
    }

    // console.log(this.attribute.path, this.parent)
  }
});

// Label
Vue.component("attribute-label", {
  model: {
    prop: "attribute",
  },
  props: {
    attribute: {
      type: Object,
    },
    onEdit: {
      default: false,
      type: Boolean,
    }, 
    path: {
      default: '',
      type: Boolean,
    },
  },
  template: `
    <template v-if="!attribute.hide_label">
      <span @dblclick="$root.editAttributeLabel(attribute)"
        :title="onEdit ? attribute.name : null"
        style="user-select: none;">
        <span class="attribute-label-html" v-html="labelText"></span>
        <span v-if="attribute.symbol">[<b style="color: red;">{{attribute.symbol}}</b>]</span>
      </span>
    </template>
  `,
  computed :{
    labelText: {
      get(){
        
        let objectPath = attributeTreePath(this.path);
        // console.log('objectPath', objectPath, this.path);
        let parent = objectPath.reverse().find(o => { 
          return o && (o.access == 'Fieldset' ||
          o.access == 'Block' ||
          o.access == 'Column' ||
          o.access == 'Group') && o.hide_by_label
        })
        
        if (parent){
          // console.log('labelText.get', this.attribute.formlabel, this.path);
          // console.log('labelText.hide label', parent);
          var prefix;
          if (parent.access == 'Fieldset' && parent.legends && parent.legends[0]){
            let legend = parent.legends[0]
            if(this.attribute != legend){
              prefix = legend.formlabel
            }
          } else {
            prefix = parent.name;
          }

          return this.attribute.formlabel.replace(prefix, '');
        }
        
        return this.attribute.formlabel;
      }
    }
  },
  watch:{
    path: function (n, o) {
      console.log('change path', n, o)
    }
  },
  mounted: function(){
    // console.log('lable mounted', this.attribute.formlabel);
  },
  created: function(){
    // console.log('lable created', this.attribute.formlabel)
  }
});

// Input
Vue.component("attribute-input", {
  model: {
    prop: "attribute",
    event: "focus",
    event: "change",
  },
  props: {
    attribute: {
      default: {},
      type: Object,
    },
    suggestions: {
      type: Array,
    },
    value: {
      type: [String, Number, Boolean, Object],
    },
    display_value: {
      type: [String, Number, Boolean, Object],
    },
    disabled: {
      type: Boolean,
    },
    reverse: {
      default: false,
      type: Boolean,
    }, 
    onEdit: {
      default: false,
      type: Boolean,
    },
  },
  data: function () {
    return {
      optionImageSizes: {
        tiny: {
          padding: "2px 5px !important",
          imageSize: {
            height: "28px",
            "max-width": "32px",
          },
        },
        mini: {
          padding: "2px 5px !important",
          imageSize: {
            height: "36px",
            "max-width": "48px",
          },
        },
        small: {
          padding: "5px 7px !important",
          imageSize: {
            height: "52px",
            "max-width": "64px",
          },
        },
        medium: {
          padding: "5px 10px !important",
          imageSize: {
            height: "68px",
            "max-width": "72px",
          },
        },
        default: {
          padding: "7px 15px !important",
          imageSize: {
            height: "76px",
            "max-width": "96px",
          },
        },
      },
      optionImageSize () {
        if(this.attribute.option_image_size){
          return this.optionImageSizes[this.attribute.option_image_size].imageSize;
        } else {
          return this.optionImageSizes['small'].imageSize;
        } 
      }
    };
  },
  template: `
  <span class="attribute-input">
    <table>
      <tr>
        <td>
          <template>
            <el-tooltip :ref="attribute.name + 'Tooltip'"
              hide-after="0" effect="light" placement="top-end"
              :disabled="!attribute.tooltip">

              <template slot="content">
                <div :style="{width: attribute.tooltip ? attribute.tooltip.width + 'px' : 'auto', height: 'auto'}">
                  <div v-if="attribute.tooltip" v-html="attribute.tooltip.code"></div>
                </div>
              </template>

              <span :style="{width: (attribute.set_input_width ? attribute.input_width + 'px' : ''), display: 'block'}"
                @mouseover="$emit('mouseover', $event)"
                @mouseleave="$emit('mouseleave', $event)"
              >

                <template v-if="attribute.access == 'TEXTBOX'">
                  <template v-if="attribute.display_type == 'SLIDER'">
                    <el-slider v-model="attribute.display_value"
                      @change="$emit('change', attribute, true)"
                      @input="$emit('input', attribute, true)"
                      :disabled="disabled"
                      :min="attribute.slidermin" :max="attribute.slidermax"
                      :step="attribute.sliderstep"
                      :format-tooltip="$root.convertDisplayValue(attribute, attribute.display_value)">
                    </el-slider>
                  </template>
              
                  <template
                    v-else-if="attribute.units == 'INTEGER' || attribute.units == 'FLOAT'">
                    <el-input-number v-model="attribute.display_value"
                      @change="$emit('change', attribute)"
                      :disabled="disabled"
                      @keyup.enter.native="$emit('submit')"></el-input-number>
                  </template>
              
                  <template v-else>
                    <el-autocomplete v-model="attribute.display_value"
                      :fetch-suggestions="querySuggestions"
                      @change="$emit('change', attribute)"
                      :disabled="disabled"
                      @select="$emit('change', attribute)"
                      @keyup.enter.native="$emit('submit')" size="mini">
                    </el-autocomplete>
                  </template>
                </template>
              
                <template v-else-if="attribute.access == 'VIEW'">
                  <el-input v-model="attribute.display_value" readonly class="attribute-readonly">
                  </el-input>
                </template>
              
                <!-- List -->
                <template v-else-if="attribute.access == 'LIST'">
                  <template v-if="attribute.display_type == 'CHECKBOX'">
                    <el-checkbox v-model="attribute.value" 
                      true-label="0"
                      false-label="1" 
                      @change="$emit('change', attribute)"
                      :disabled="disabled">
                      <template v-if="reverse">
                        {{attribute.formlabel}}
                        <!-- <label class="el-form-item--mini el-form-item__label"></label> -->
                        <!-- <attribute-label v-model="attribute" :on-edit="onEdit"></attribute-label> -->
                      </template>
                    </el-checkbox>
                  </template>
              
                  <!-- Switch -->
                  <template v-else-if="attribute.display_type == 'SWITCH'">
                    <el-switch v-model="attribute.value"
                      :active-value="activeValue()"
                      :inactive-value="inactiveValue()"
                      :active-text="attribute.option_hide_text ? '' : activeText()"
                      :inactive-text="attribute.option_hide_text ? '' : inactiveText()"
                      :title="attribute.option_hide_text ? inactiveText() + '/' + activeText() : ''"
                      inactive-color="#c8c8c8"
                      :disabled="disabled"
                      @change="$emit('change', attribute)"
                      :style="{float: reverse ? 'right' : ''}"
                      >
                    </el-switch>
                  </template>
              
                  <!-- Radio -->
                  <template v-else-if="attribute.display_type == 'RADIO'">
                    <el-radio-group v-model="attribute.value" size="mini"
                      :class="attribute.radiostyle == 'Image' ? 'image-radio' : ''"
                      :disabled="disabled"
                      @change="$emit('change', attribute)">
                      <template v-if="!attribute.radiostyle || attribute.radiostyle == 'DEFAULT'">
                        <el-radio v-for="(v, n) in attribute.options"
                          :label="v">{{n}}</el-radio>
                      </template>
                      <template v-else-if="attribute.radiostyle == 'BORDER'">
                        <el-radio v-for="(v, n) in attribute.options" border
                          :label="v">{{n}}</el-radio>
                      </template>
                      <template v-else-if="attribute.radiostyle == 'BUTTON'">
                        <el-radio-button v-for="(v, n) in attribute.options"
                          :label="v">{{n}}</el-radio-button>
                      </template>
                      <template v-else-if="attribute.radiostyle == 'Image'">
                        <!-- <slot name="radio-image"></slot> -->
                        <el-radio v-for="(v, n) in attribute.options" :label="v" :key="n"
                          :style="{
                            padding: attribute.option_image_size ? optionImageSizes[attribute.option_image_size].padding : optionImageSizes['small'].padding
                          }"
                          border>
            
                          <el-image
                            :src="(attribute.image_options && attribute.image_options[v]) ? attribute.image_options[v].base64 : ''"
                            :atl="n" fit="scale-down"
                            @dblclick="onEdit ? $root.addOptionImage(attribute, v) : console.log(onEdit)"
                            :style="attribute.option_image_size ? optionImageSizes[attribute.option_image_size].imageSize : optionImageSizes['small'].imageSize">
            
                            <div slot="error"><i class="fa fa-image fa-2x"></i></div>
                          </el-image>
              
                          <div v-if="attribute.option_hide_text">{{n}}</div>
                        </el-radio>
                      </template>
                    </el-radio-group>
                  </template>
              
                  <template v-else-if="$root.isMaterial(attribute)">
                    <el-select v-model="attribute.value"
                      placeholder="Select"
                      :no-match-text="$root.noMatchMaterialValue(attribute)"
                      no-data-text="No data"
                      :disabled="disabled"
                      class="image-select"
                      :style="{height: attribute.option_image_size ? optionImageSizes[attribute.option_image_size].imageSize.height : ''}"
                      @change="$emit('change', attribute)">
              
                      <template slot="prefix">
                        <el-image
                          :src="$root.data.materials[attribute.value] ? $root.data.materials[attribute.value].thumbnail : ''"
                          atl="Img" fit="scale-down"
                          :class="attribute.option_image_size + ' image-thumbnail'">
                          <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                        </el-image>
                        <span v-if="!attribute.option_hide_text" style="padding-left: 5px; color: unset;">
                          {{getKeyByValue(attribute.options, attribute.value)}}
                        </span>
                      </template>

                      <template v-if="$root.fullMaterials">
                        <el-option v-for="(name, item) in $root.data.materials"
                          :key="item.value" :label="item.label"
                          :value="item.value">
                          <img :src="item.thumbnail"
                            style="height: 22px;width: 22px;"
                            class="thumbnail-inlist">
                          {{item.label}}
                      </template>

                      <template v-else>
                        <el-option v-for="(v, label) in attribute.options" class="image-option"
                          :key="v" :value="v" :label="label">
              
                          <template v-for="mat in $root.numMaterials(v)">
                              <!-- <img v-if="$root.data.materials[mat]"
                                :src="$root.data.materials[mat] ? $root.data.materials[mat].thumbnail : ''"
                                style="height: 22px; width:22px;"
                                class="thumbnail-inlist"> -->

                                <el-image v-if="$root.data.materials[mat]"
                                  :src="$root.data.materials[mat] ? $root.data.materials[mat].thumbnail : ''"
                                  atl="Img" fit="scale-down"
                                  :class="attribute.option_image_size + ' image-thumbnail'">
                                  <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                                </el-image>
                                <span style="padding-left: 5px; color: unset;">{{label}}</span>
                          </template>
                      </template>
                      </el-option>
                    </el-select>
                  </template>
              
                  <template v-else>
                    <el-select v-model="attribute.value" 
                      placeholder=""
                      :disabled="disabled"
                      @change="$emit('change', attribute)" 
                      size="mini"
                      :style="{height: attribute.option_image_size ? optionImageSizes[attribute.option_image_size].imageSize.height : ''}"
                      :class="attribute.option_use_image ? 'image-select' : ''">
      
                      <template slot="prefix">
                        <template v-if="attribute.option_use_image">
                          <el-image
                            :src="attribute.image_options ? attribute.image_options[attribute.value].base64 : ''"
                            atl="Img" fit="scale-down"
                            :class="attribute.option_image_size + ' image-thumbnail'">
                            <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                          </el-image>
                          <span v-if="!attribute.option_hide_text" style="padding-left: 5px;">
                            {{getKeyByValue(attribute.options, attribute.value)}}
                          </span>
                        </template>
                      </template>
      
                      <el-option v-for="(v, n) in attribute.options" :key="v"
                        :label="n" :value="v" :class="attribute.option_use_image ? 'image-option' : ''">
                        <template v-if="attribute.option_use_image">
                          <el-image
                            :src="attribute.image_options ? attribute.image_options[v].base64 : ''"
                            atl="Img" fit="scale-down"
                            :class="attribute.option_image_size + ' image-thumbnail'">
                            <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                          </el-image>
                        </template>
                        <span style="padding-left: 5px;">{{n}}</span>
                        <!-- <template v-if="onEdit">
                          <span style="float: right;" @click="$event.stopPropagation(); console.log(attribute)"><i class="fa fa-pencil"></i></span>
                        </template> -->
                      </el-option>
                    </el-select>
                  </template>
                </template>
              </span>
            </el-tooltip>
          </template>
        </td>
      </tr>
    </table>

  </span>
  `,
  methods: {
    querySuggestions(queryString, cb) {
      console.log(this.attribute.suggestions);
      if (this.attribute.suggestions) {
        cb(
          this.attribute.suggestions.map((value) => {
            return {
              value: value,
            };
          })
        );
      } else {
        cb([]);
      }
    },
    getKeyByValue(object, value) {
      return Object.keys(object).find(key => object[key] === value);
    },
    editSelectOption(queryString, cb) {
      console.log(this.attribute.suggestions);
      if (this.attribute.suggestions) {
        cb(
          this.attribute.suggestions.map((value) => {
            return {
              value: value,
            };
          })
        );
      } else {
        cb([]);
      }
    },
    activeValue() {
      return this.attribute.options[this.activeText()];
    },
    activeText() {
      var t;
      if(this.attribute.active_value) {
        t = Object.keys(this.attribute.options).find(l=>{
          return this.attribute.options[l] == this.attribute.active_value;
        })
        if(!t) {
          this.attribute.active_value = Object.values(this.attribute.options)[0];
        }
      }
      if(!t) t = Object.keys(this.attribute.options)[0];

      return t;
    },
    inactiveValue(){
      return this.attribute.options[this.inactiveText()];
    },
    inactiveText(){
      return Object.keys(this.attribute.options).find(l=>{
        return l != this.activeText();
      })
    },
  },
  watch: { 
    // attribute: {
    //   handler: function (newVal, oldVal) { 
    //     console.log('attribute changed: ', newVal, ' | was: ', oldVal)
    //     this.$forceUpdate();
    //   },
    //   deep: true
    // },
    value: function(newVal, oldVal){
      console.log('value changed: ', oldVal , ' => ', newVal)
    }, 
    display_value: function(newVal, oldVal){
      console.log('display_value changed: ', oldVal , ' => ', newVal)
    },
  },
  computed: {
    display_value: {
      // getter
      get: function () {
        // console.log('get display_value: '+ this.attribute.formlabel, this.attribute.display_value)
        // return this.attribute.display_value + 'xxx'
      },
      // setter
      set: function (newValue) {
        // console.log('set display_value: '+ this.attribute.formlabel + '  ' + newValue)
        // this.attribute.value  = newValue + 'ZZZ'
      }
    }
  },
});

// Display Types
Vue.component("display-type", {
  model: {
    prop: "attribute",
    event: "change",
  },
  props: {
    attribute: {
      type: Object,
    },
   
    default_types: {
      default: true,
      type: Boolean,
    },
  },
  data: function () {
    return {
      display_type: [
        {
          label: "Default",
          value: "DEFAULT",
        },
        {
          label: "Materials",
          value: "MATERIAL",
        },
        {
          label: "Checkbox",
          value: "CHECKBOX",
        },
        {
          label: "Switch",
          value: "SWITCH",
        },
        {
          label: "Radio",
          value: "RADIO",
        },
        {
          label: "Slider",
          value: "SLIDER",
        },
      ],
      
    }
  },
  template: `
  <span>
    <el-select v-model="attribute.display_type"
      placeholder="Default"
      @change="$emit('change', attribute)">

      <el-option v-for="type in display_type" :label="type.label"
        :value="type.value"
        :disabled="!(validDisplayType(attribute, type.value))">

        <span style="float: left">{{ type.label }}</span>

        <span style="float: right; width: 40%;">
          <template v-if="type.label == 'Default'">
            <span>{{attribute.access}}</span>
            <!-- <template v-if="attribute.access == 'TEXTBOX'">
              <el-input size="mini"></el-input>
            </template>
            <template v-else-if="attribute.access == 'LIST'">
              <el-select size="mini"></el-select>
            </template>
            <template v-else><span>{{attribute.access}}</span></template> -->
          </template>
          <template v-else-if="type.label == 'Materials'">
            <img v-if="$root.defaultMaterial()"
              :src="$root.defaultMaterial().thumbnail"
              style="height: 22px; width:22px;"
              class="thumbnail-inlist">
          </template>
          <template v-else-if="type.label == 'Checkbox'">
            <el-checkbox v-model="default_types">
            </el-checkbox>
          </template>
          <template v-else-if="type.label == 'Switch'">
            <el-switch width="40" v-model="default_types">
            </el-switch>
          </template>
          <template v-else-if="type.label == 'Radio'">
            <el-radio v-model="default_types" :label="true">A
            </el-radio>
            <el-radio v-model="default_types" :label="false">B
            </el-radio>
          </template>
          <template v-else-if="type.label == 'Slider'">
            <el-slider :value="50"></el-slider>
          </template>
        </span>
      </el-option>
    </el-select>

    <slot></slot>

  </span>
  `,
  methods: {
    validDisplayType(attribute, type) {
      // console.log(attribute, type);
      if (type == "SLIDER") {
        if (
          attribute.units == "INTEGER" ||
          attribute.units == "FLOAT" ||
          attribute.units == "DEGREES" ||
          attribute.units == "PERCENT"
        ) {
          return true;
        } else {
          return false;
        }
      } else if (type == "CHECKBOX") {
        if (attribute.access != "LIST") {
          return false;
        }
        return isBoolean(attribute);
      } else if (type == "SWITCH") {
        if (attribute.access != "LIST") {
          return false;
        }
        return isSwitch(attribute);
      } else if (type == "MATERIAL") {
        if (attribute.access != "LIST") {
          return false;
        }
        if (isBoolean(attribute)) {
          return false;
        }
      } else if (type == "RADIO") {
        return attribute.access == "LIST";
      }
      return true;
    },
  },
  computed: {
  },
});

// Edit Attribute
Vue.component("edit-attribute", {
  model: {
    prop: "attribute",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    },
    onEdit: {
      default: false,
      type: Boolean,
    }, 
    inline: {
      default: false,
      type: Boolean,
    },
  },
  template: `
    <span>
      <span class="action-icon edit-attribute" v-if="onEdit"
        @click="$root.editAttribute(attribute)">
          <i class="fa fa-pencil" aria-hidden="true"></i>
      </span>
      <template v-if="$root.isCustomAttribute(attribute) && onEdit">
        <div class="action-icon edit-attribute" :style="{display: (inline ? 'inline-block' : 'block')}"
          @click="removeAttr()">
          <i class="fa fa-trash"></i>
        </div>
      </template>
    </span>
  `,
  methods:{
    removeAttr() {
      this.$confirm('Delete?', 'Warning', {
        confirmButtonText: 'OK',
        cancelButtonText: 'Cancel',
        type: 'warning'
      }).then(() => {
        this.$root.removeCustomAttribute(this.attribute, this.parent)
        this.$message({
          type: 'success',
          message: 'Delete completed'
        });
      }).catch(() => {
        this.$message({
          type: 'info',
          message: 'Delete canceled'
        });          
      });
    }
  }
});

// Handle Attribute
Vue.component("handle-attribute", {
  model: {
    prop: "attribute",
  },
  props: {
    attribute: {
      type: Object,
    },
  },
  template: `
    <span class="handle-attribute action-icon">
      <i class="fa fa-bars" aria-hidden="true"></i>
    </span>
  `
});

// Custom Attribute
Vue.component("custom-attribute", {
  model: {
    prop: "attribute",
    event: "change",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    }, 
    group: {
      type: Object,
    },
    layout: {
      type: Object,
    },
    disabled: {
      default: false,
      type: Boolean,
    },
    onEdit: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
    image_input_height: {
      default: 'auto',
      type: String,
    },
    zindex: {
      default: 100,
      type: Number,
    },
  },
  data: function () {
    return {
    }
  },
  template: `
  <span>
    <template v-if="attribute.access == 'GRID'">
      <fieldset-attribute v-model="attribute"
        :ref="attribute.path"
        :on-edit="onEdit"
        :display_action="display_action"
        >
        <span>content</span>
      </fieldset-attribute>
    </template>
    <template v-if="attribute.access == 'Fieldset'">
      <fieldset-attribute v-model="attribute"
        :ref="attribute.path"
        :parent="parent"
        :disabled="disabled"
        :on-edit="onEdit"
        :display_action="display_action"
        >
        <span>content</span>
      </fieldset-attribute>
    </template>
  
    <template v-if="attribute.access == 'Divider'">
      <div :style="{margin: attribute.formlabel != '' ? '10px 0' : '5px 0'}">
        <el-divider class="section-attribute"
          :direction="attribute.direction"
          :content-position="attribute.label_position">
          <span v-if="attribute.formlabel != ''" 
            @dblclick="$root.editAttributeLabel(attribute)"
            class="subtitle"
            v-html="attribute.formlabel">
          </span>
        </el-divider>
      </div>
    </template>

    <template v-if="attribute.access == 'Section'">
      <el-divider content-position="left" class="section-attribute">{{attribute.name}}</el-divider>
    </template>

  </span>
  `,
  methods: {
    insertField(e){
      console.log(e);
      console.log(this);
    },
    dividerClass(){
      a = this.attribute.name != '' ? 'section-attribute' : ''
      console.log(a)
      return this.attribute.name != '' ? 'section-attribute' : '';
    }
  },
  computed: {
    dividerStyle(){
      if (this.attribute.name != ''){
        return {margin: '10px 0'}
      } else {
        return {margin: '5px 0'}
      }
    }
  },
  mounted: function (){
  }
});

// Image Attribute
Vue.component("image-attribute", {
  model: {
    prop: "attribute",
    event: "change",
  },
  props: {
    attribute: {
      default: {},
      type: Object,
    },
    parent: {
      type: Object,
    },
    group: {
      type: Object,
    },
    layout: {
      type: Object,
    },
    onEdit: {
      default: false,
      type: Boolean,
    }, 
    disabled: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
    grid: {
      default: [1, 1],
      type: Array,
    },
    position: {
      default: '',
      type: String,
    }, 
    update_pos: {
      default: false,
      type: Boolean,
    },
    zindex: {
      default: 100,
      type: Number,
    },
  },
  data: function () {
    return {
    }
  },
  template: `
    <div @dblclick="insertField($event)" v-on:keyup.up="moveUp($event)">
    <vue-draggable-resizable v-for="(inputGroup, index) in attribute.attributes" :key="inputGroup.key"
      :w="inputGroup.w" :h="inputGroup.h"
      :x="inputGroup.x" :y="inputGroup.y"
      :min-width="50"
      :ref="inputGroup.key"
      :z="100"
      :handles="['mr','ml', 'bm']"
      :parent="true" 
      :grid=grid
      :drag-handle="'.drag-handle'" 
      class-name="image-input-group"
      class-name-active="image-input-active"
      :style="draggableResizableStyle"
      @activated="onActivated(inputGroup)" @deactivated="onDeactivated(inputGroup)"
      @dragging="onDrag" @resizing="onResize" 
      @resizestop="(x, y, width, height) => {onResizeStop(x, y, width, height, inputGroup)}"
      @dragstop="(x, y) => {onDragStop(x, y, inputGroup)}"
      :enable-native-drag="true">
        <div class="image-input-action">
          <span class="drag-handle action-icon" v-if="onEdit || attribute.on_edit">
            <i class="fa fa-thumb-tack" aria-hidden="true"></i>
          </span>

          <span class="action-icon" v-if="onEdit || attribute.on_edit"
            @click="removeInputGroup(inputGroup, index)"
            style="float: right;">
            <i class="fa fa-trash" aria-hidden="true"></i>
          </span>
        </div>

        <draggable tag="el-form" handle=".handle-attribute"
          :list="inputGroup.attributes" 
          @add="changeInputs(inputGroup)"
          @remove="changeInputs(inputGroup)"
          @update="changeInputs(inputGroup)"
          :component-data="{
              attrs:{
                'disabled': disabled,
                'label-width': 'auto',
                'size': 'mini',
                'class': 'form-layout',
              }
            }"
          v-bind='{group: "layout.attributes"}'>

          <div v-for="(a, attr_index) in inputGroup.attributes" :key="a.name || a.key" 
          :class="'attribute-item ' + attribute.access + (onEdit ? ' on-edit' : '') + ' image-float-attribute-item'"
          :style="$root.attributeItemStyle">
            <attribute
              v-model="a"
              :disabled="disabled ? disabled : $root.attributeDisabled(a)"
              :parent="inputGroup"
              :on-edit="onEdit"
              :display_action="display_action"
            ></attribute>
          </div>

          <el-form-item v-if="inputGroup.attributes.length == 0" label="">
            <div style="border: 1px dashed darkgrey; height: 32px; padding: 0 5px;">No Attribute</div>
          </el-form-item>
        </draggable>
      </vue-draggable-resizable>

      <div :style="{width: '100%', display: 'flex', 'justify-content': attribute.position, 'z-index': 1, 'user-select': 'none'}">
        <el-image
          @load="onLoadImage($event)"
          :style="imageSize"
          :src="attribute.image.base64" fit="contain">
        </el-image>
      </div>
    </div>
  `,
  methods: {
    onLoadImage (e) {
      if (this.attribute.image.naturalWidth) return;

      this.attribute.image.naturalWidth = e.target.naturalWidth;
      this.attribute.image.naturalHeight = e.target.naturalHeight;

      console.log('onLoadImage')

      if (!this.attribute.width){
        if(this.attribute.image.naturalWidth > 500){
          this.attribute.width = 250;
        } else {
          this.attribute.width = this.attribute.image.naturalWidth;
          this.attribute.height = this.attribute.image.naturalHeight;
        }
      }
    },
    moveUp (e) {
      console.log(e)
      e.stopPropagation()
    },
    onActivated (inputGroup) {
      inputGroup.active = true;
      console.log(inputGroup.active)
    }, 
    onDeactivated (inputGroup) {
      inputGroup.active = false;
      console.log(inputGroup.active)
    },
    insertField(e){
      console.log(e);
      if(!this.onEdit){
        return
      }

      block = {
        access: 'FloatInput',
        key: parse_uuid(),
        x: e.offsetX,
        y: e.offsetY,
        w: 150,
        h: 'auto',
        attributes: []
      }

      this.attribute.attributes.push(block)
    },

    removeInputGroup(inputGroup, index){
      // move floatinput to parent of image attribute
      this.$root.removeCustomAttribute(inputGroup, this.parent);

      // remove floatinput from image attributes
      this.$delete(this.attribute.attributes, index);
    },

    changeInputs(inputGroup){
      console.log('changeInputs', inputGroup)
      this.updateBorderHeight(inputGroup)
    },

    imagePosition() {
      c_img = this.$children.find(c=>{ return c.$options._componentTag == 'el-image'; })
      let position = [c_img.$el.offsetLeft, c_img.$el.offsetTop]
      return position
    },
    onResize: function (x, y, width, height) {
      console.log(x, y, width, height)
      this.grid = [5, 5];
    }, 
    onResizeStop: function (x, y, width, height, inputGroup) {
      console.log('onResizeStop')

      inputGroup.x = x;
      inputGroup.y = y;
      inputGroup.w = width;
      inputGroup.h = this.updateBorderHeight(inputGroup);

      this.grid = [1, 1];

      this.pinBlockPosition(inputGroup)
    },
    onDrag: function (x, y) {
      console.log(this)
      console.log(x, y)
    },
    onDragStop: function (x, y, inputGroup) {
      console.log('onDragStop')
      inputGroup.x = x;
      inputGroup.y = y;
      this.pinBlockPosition(inputGroup)
    },
  
    pinBlockPosition: function (block) {
      console.log('pinBlockPosition()', [block.x, block.y], this.imagePosition())
      block.pin_position = this.imagePosition();
    },

    updateBorderHeight(inputGroup){
      let self = this.$refs[inputGroup.key][0]
      self.$el.style.height = 'auto'
      
      var style = window.getComputedStyle(self.$el);
      height = parseFloat(style.getPropertyValue('height'), 10)
      
      self.height = height;
      s = self.checkParentSize();

      console.log('updateBorderHeight', 'height', height, this)
      console.log(s)
      return height;
    },

    updatePosition: function () {
      console.log('this.updatePosition()', 'update_pos', this.update_pos)
      if(this.update_pos == false){
        return;
      }
      this.update_pos = false

      const img_pos = this.imagePosition()
      // console.log('this.imagePosition()', img_pos)

      if(this.attribute.on_edit){
        return;
      }
      for (let i = 0; i < this.attribute.attributes.length; i++) {
        const block = this.attribute.attributes[i];
        // console.log('Block Position', [block.x, block.y], block.pin_position)

        const move_x = img_pos[0] - block.pin_position[0]
        // console.log('move_x', move_x)
        if(move_x != 0){
          block.x += move_x;
          this.pinBlockPosition(block);
        }
      }
    },
  },
  watch: { 
    onEdit: function(newVal, oldVal){
      console.log('onEdit changed: ', newVal)
      this.update_pos = true;
    },
    on_edit: function(newVal, oldVal){
      console.log('onEdit changed: ', newVal)
      this.update_pos = true;
    }, 
    position: function(newVal, oldVal){
      console.log('position changed: ', newVal)
    },
    display_action: function(newVal, oldVal){
      console.log('display_action changed: ', newVal)
      this.update_pos = true;
    }, 
  },
  computed: {
    imageSize(){
      // console.log((this.attribute.height/this.attribute.image.naturalHeight) * this.attribute.image.naturalWidth + 'px')
      // console.log(this.attribute.height, this.attribute.image.naturalHeight , this.attribute.image.naturalWidth , 'px')
      // console.log(this.attribute)
      return {
        border: (this.onEdit || this.attribute.on_edit) ? '1px solid gray' : 'none',
        width: this.attribute.width + 'px',
        height: (this.attribute.width/this.attribute.image.naturalWidth) * this.attribute.image.naturalHeight + 'px'
      }
    },
    draggableResizableStyle(){
      if(this.onEdit || this.attribute.on_edit){
        return {
          // border: '1px solid darkgrey',
          background: 'rgba(245, 245, 245, 0.59)'
        }
      } else{
        return {
          // border: 'rgba(245, 245, 245, 0.59)',
          background: 'none'
        }
      }
    },
  },
  updated: function (){
    this.$nextTick(function () {
      // Code that will run only after the
      // entire view has been re-rendered
      this.updatePosition()
    })
    
  },
  mounted: function (){
    if(!this.attribute.on_edit){
      if(!this.parent.path){
        this.parent.path = ""
      }
  
      this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`
  
      for (let i = 0; i < this.attribute.attributes.length; i++) {
        const a = this.attribute.attributes[i];
        a.path = this.attribute.path + '/' + `${this.attribute.access}::${this.attribute.key}`;
      }
    }

    var self = this;
    this.attribute.onEdited = function(e) {
      console.log('onEdited', e)
      self.update_pos = true;
      self.updatePosition()
    }

    this.position = this.attribute.position;

    this.$nextTick(() => {
      window.addEventListener('resize', () => {
        this.update_pos = true;
        this.updatePosition()
        console.log('resize', this.update_pos, this.imagePosition())
      });

      window.addEventListener('keydown', (event) => {
        console.log('keydown', event.keyCode)
        if (![37, 38, 39, 40].includes(event.keyCode)){
          return;
        }

        activeBlocks = this.attribute.attributes.filter(block=>{
          return block.active;
        })

        if(activeBlocks.length == 0){
          return;
        } else {
          event.stopPropagation();
        }

        console.log('activeBlocks', activeBlocks)

        if (event.keyCode == 37) {
          // left = 37
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.x -= this.grid[0]
          }
        } 
        else if (event.keyCode == 38){
          // up = 38
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.y -= this.grid[1]
          }
        }
        else if (event.keyCode == 39){
          // right = 39
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.x += this.grid[0]
          }
        }
        else if (event.keyCode == 40){
          // down = 40
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.y += this.grid[1]
          }
        }
      });
    })
  }
});

// Group
// Attribute
Vue.component("block-attribute", {
  model: {
    prop: "attribute",
    event: "change",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    },
    disabled: {
      default: false,
      type: Boolean,
    }, 
    reverse: {
      default: false,
      type: Boolean,
    }, 
    onEdit: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
  },
  template: `
  <div class="block-attribute" :class="onEdit ? 'on-edit' : ''">
    <el-card shadow="never" :style="{'background-color': attribute.background_color}">
      <div slot="header" v-if="onEdit || attribute.name != ''"
        class="block-attribute-header">
        <span>
          <handle-attribute v-if="onEdit && display_action"></handle-attribute>

          <!-- <span>Card name</span> -->
          <span class="subtitle" @dblclick="editName()">
            <template v-if="attribute.name == ''">
              <span style="color: #909399">Block Title</span>
            </template>
            <template v-else>
              <span v-html="attribute.name"></span>
            </template>
          </span>
        </span>

        <template v-if=onEdit>
          <div>
            <el-select v-model="attribute.num_col" class="select-icons" style="width: 50px;"
              @change="changeNumberColumns" placeholder="" size="mini" :disabled="false">
              <template slot="prefix">
                <i v-for="i in attribute.num_col" :key="i" class="fa fa-square-o"></i>
              </template>
              <!-- <template slot="suffix">
                <i v-for="i in attribute.num_col" :key="i" class="fa fa-square-o"></i>
              </template> -->
    
              <el-option :value="1" :label="' '"> <i class="fa fa-square-o"></i> </el-option>
              <el-option :value="2" :label="' '"> <i class="fa fa-square-o"></i> <i class="fa fa-square-o"></i> </el-option>
            </el-select>
            <edit-attribute v-model="attribute" :parent="parent" :on-edit="onEdit" :inline="true">
            </edit-attribute
          </div>
        </template>
      </div>

      <el-row :gutter="10" style="display: flex;">
        <template v-for="(column, index) in attribute.attributes">
          <el-col :span="colWidth(column)" :key="index"
            :ref="column.key"
            :style="columnWidth(column, index)"
          >
            <template v-if="column.name != ''" >
              <h4 style="margin: 0;"><span class="subtitle" v-html="column.name"></span></h4>
            </template>

            <draggable tag="el-form" handle=".handle-attribute"
              :list="column.attributes" 
              :component-data="{
                  attrs:{
                    'disabled': disabled,
                    'label-width': 'auto',
                    'label-position': column.labelposition,
                    'size': 'mini',
                    'class': 'form-layout',
                  }
                }"
              @add="changeItems(column)"
              @remove="changeItems(column)"
              @update="changeItems(column)"
              v-bind='{group: "layout.attributes"}'>
              
              <div v-for="(a, attr_index) in column.attributes" :key="a.name || a.key" 
              :class="'attribute-item ' + a.access + (onEdit ? ' on-edit' : '')"
              :display_type="a.display_type"
              :style="$root.attributeItemStyle" 
                v-if="$root.attributeVisible(a) && (a.render || a.render == undefined)"
                >
    
                <attribute
                  v-model="a"
                  :parent="column"
                  :on-edit="onEdit"
                  :display_action="display_action"
                  :disabled="disabled ? disabled : $root.attributeDisabled(a)"
                  @change="$root.changeAttrValue"
                  @input="$root.onChangeAttrValue(a)"
                  @submit="$root.submit"
                ></attribute>
              </div>
    
              <el-form-item v-if="column.attributes.length == 0" label="">
                <div style="height: 32px; padding: 0 5px; border: 1px dashed gray;">No Attribute</div>
              </el-form-item>
  
              <div v-if="onEdit" slot="footer">
                <el-form size="mini" :disabled="false" >
                  <el-button @click="$root.editLayout(column, refSize(column.key))"
                    size='mini' class="icon-button">
                    <i class="fa fa-sliders fa-2x"></i>
                  </el-button>
                </el-form>
              </div>
            </draggable>
          </el-col>
        </template>
      </el-row>
      <div v-if="showSliderSize()"> 
        <el-slider v-model="attribute.attributes[0].span" @input="changeSpan($event)"
          :min="0" :max="24" :step="1" show-stops :show-tooltip="false"></el-slider>
      </div>
    </el-card>
  </div>
  `,
  methods: {
    editName(){
      if (!this.onEdit) return;

      this.$prompt('Enter title', 'Block Title', {
        inputValue: this.attribute.name,
        inputPlaceholder: 'Title',
        confirmButtonText: 'OK',
        cancelButtonText: 'Cancel',
      }).then(({ value }) => {
        this.attribute.name = value;
      }).catch(() => {
      });
    },
    refSize(ref){
      return this.$refs[ref][0];
    },
    changeItems(column){
      console.log(this.attribute, column)
      // a = column.attributes[0]
      // if (a) console.log(attributeTreePath(a.path));
    },   
    showSliderSize(){
      return this.onEdit && 
        this.attribute.attributes.length == 2 && 
        !this.attribute.attributes.find(c=>{ return c.fixed_width });
    },   
    columnWidth(column, index){
      // Nếu 1 trong 2 fixed_width thì col còn lại là flex_grow = 1
      if(this.attribute.attributes.length == 1){
        return {};
      }

      var style = {};

      if(column.fixed_width){
        style.width = column.width + 'px';
        style['flex-shrink'] = 0;
      } else {
        let otherColumn = this.otherColumn(column);
        // console.log(otherColumn)
        if(otherColumn && otherColumn.fixed_width){
          style['flex-grow'] = 1;
        } else {
          // Use span prop
          // return {};
        }
      }

      if (column.use_min_width) style['min-width'] = column.min_width + 'px';
      if (column.use_max_width) style['max-width'] = column.max_width + 'px';
      return style;
    },
    otherColumn(column){
      return this.attribute.attributes.find(c=>{ return c.key != column.key });
    },
    changeNumberColumns() {
      if(this.attribute.num_col == 1){
        col2 = this.attribute.attributes[1]
        if(col2){
          for (let i = 0; i < col2.attributes.length; i++) {
            const a = col2.attributes[i];
            this.attribute.attributes[0].attributes.push(a)
          }
          this.$delete(this.attribute.attributes, 1);
        }
      } else {
        this.attribute.attributes.push(
          {
            render: true,
            access: 'Column',
            name: '',
            key: parse_uuid(),
            path: this.attribute.path + '/' + `${this.attribute.access}::${this.attribute.key}`,
            span: 12,
            attributes: []
          }
        )
      }

      this.$forceUpdate()
    },
    changeSpan(span) {
      if(this.attribute.num_col == 1){
      } else {
        console.log(span)
        if(this.attribute.attributes[1]){
          this.attribute.attributes[1].span = 24 - span
        }
      }
    },
    colWidth(column) {
      return this.attribute.attributes.length == 1 ? 24 : (column.span == 0 ? 24 : column.span);
    }
  },
  watch: {
    // disabled(n, o){
    //   console.log(this.attribute.access, 'change disable', n, '=>', o)
    // }
  },
  computed: {

  },
  mounted: function (){
    // console.log('attribute mounted', this.attribute.access, this.disabled)
    var self = this;

    this.attribute.vaildSize = function(path) {
      console.log('columnWidth', self, this, path)
      return path;
    }

    
  },
  created: function(){
    // console.log('attribute create', this.attribute.access, this.key)
    
    if (!this.attribute.on_edit){
      if (!this.parent.path) {
        this.parent.path = ""
      }
  
      this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`
  
      for (let i = 0; i < this.attribute.attributes.length; i++) {
        const col = this.attribute.attributes[i];
        col.path = this.attribute.path + '/' + `${this.attribute.access}::${this.attribute.key}`;
      }
    }

    // console.log(this.attribute.path, this.parent)
  }
});

// Fieldset
Vue.component("fieldset-attribute", {
  model: {
    prop: "attribute",
    event: "change-attributes",
    event: "change-legend",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    },
    group: {
      type: Object,
    },
    layout: {
      type: Object,
    },
    index: {
      type: Number,
    },
    onEdit: {
      default: false,
      type: Boolean,
    }, 
    disabled: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
  },
  data: function () {
    return {
    };
  },
  template: `
  <span>
    <el-card shadow="never" :style="{'background-color': attribute.background_color}">
      <!-- <Legend></Legend> -->
      <div>
        <draggable tag="el-form" handle=".handle-attribute"
          :list="attribute.legends" 
          @remove="changeLegend"
          @add="changeLegend"
          @update="changeLegend"
          :component-data="{
              attrs:{
                'disabled': disabled,
                'label-width': 'auto',
                'size': 'mini',
                'class': 'form-layout',
              }
            }"
          v-bind='{group: {
            name: "layout.attributes",
            pull: true,
            put(a, b, c, d) {
              // Return true to allow, return false to disallow
              // return true;
              return onPut(a, b, c, d);
            }
          }}'>
  
          <div v-for="(a, attr_index) in attribute.legends" :key="attribute.name || attribute.key" 
            :class="'attribute-item ' + a.access + (onEdit ? ' on-edit' : '')"
            :display_type="attribute.display_type"
            :style="$root.attributeItemStyle">
            <attribute
              v-model="a"
              :parent="attribute"
              :on-edit="onEdit"
              :display_action="display_action"
              reverse="true"
            ></attribute>
          </div>
  
          <el-form-item v-if="attribute.legends && attribute.legends.length == 0" label="">
            <div style="border: 1px dashed darkgrey; height: 32px; padding: 0 5px;" title="Only Checkbox Attribute!">No Legend</div>
          </el-form-item>
        </draggable>
      </div>
      
      <!-- Items -->
      <div v-if="(isLegendControlVisible && !onEdit) ? visibleItems : true">
        <el-divider></el-divider>
        <draggable tag="el-form" handle=".handle-attribute"
          :list="attribute.attributes"
          :component-data="{
              attrs:{
                'disabled': disabled ? disabled : (isLegendControlDisable ? disabledItems : false),
                'label-width': 'auto',
                'size': 'mini',
                'class': 'form-layout',
              }
            }"
          v-bind='{group: "layout.attributes"}'>
          <div v-for="(a, attr_index) in attribute.attributes" :key="a.name || a.key" class="attribute-item" :style="$root.attributeItemStyle">

            <template v-if="(isLegendControlVisible && !onEdit) ? visibleItems : true">
              <attribute
                v-model="a"
                :parent="attribute"
                :on-edit="onEdit"
                :display_action="display_action"
                :disabled="disabled ? disabled : (isLegendControlDisable ? disabledItems : false)"
              ></attribute>
            </template>

          </div>
          <el-form-item v-if="attribute.attributes.length == 0" label="">
            <div style="border: 1px dashed darkgrey; height: 32px; padding: 0 5px;">No Attribute</div>
          </el-form-item>
        </draggable>
      </div>
  </el-card>
  </span>

  `,
  methods: {
    addFieldsetLegend() {
      this.$prompt('Select attribute', 'Legend', {
        confirmButtonText: 'OK',
        cancelButtonText: 'Cancel',
        inputPlaceholder: 'Enter attribute name (Only Check or Switch Type)',
        inputErrorMessage: 'Invalid attribute',
        inputValidator(value){
          let a = this.$root.dcAttribute(value)
          checktype = a && (this.$root.isSwitch(a.options) || this.$root.isBoolean(a.options));
          console.log(a)
          return (a && checktype) ? true : false;
        }
        // inputPattern: /[\w!#$%&'*+/=?^_`{|}~-]+(?:\.[\w!#$%&'*+/=?^_`{|}~-]+)*@(?:[\w](?:[\w-]*[\w])?\.)+[\w](?:[\w-]*[\w])?/,
        // inputErrorMessage: 'Invalid Email'
      }).then(({ value }) => {
        this.attribute.legend = this.$root.dcAttribute(value);

        console.log(this.$root.dcAttribute(value))
        console.log(this.attribute)

        this.$message({
          type: 'success',
          message: 'Attribute:' + value
        });
      }).catch(() => {
        this.$message({
          type: 'info',
          message: 'Input canceled'
        });       
      });
    },
    changeLegend(e){
      console.log('changeLegend', e)
      return false;
      console.log(this.$root)
      legend = this.attribute.legends[this.attribute.legends.length-1]
      console.log(legend)

      if(this.attribute.legends.length == 2){
        this.$alert('Legend must be Checkbox.', 'Invalid Type!', {
          confirmButtonText: 'OK',
          callback: action => {
            this.$delete(this.attribute.legends, this.attribute.legends.indexOf(legend))
          }
        });
      } else if(legend){
        if(isBoolean(legend)){
        }else{
          this.$alert('Legend must be Checkbox.', 'Invalid Type!', {
            confirmButtonText: 'OK',
            callback: action => {
              this.$delete(this.attribute.legends, this.attribute.legends.indexOf(legend))
            }
          });
        }
      }
    },
    onPut: function(a, b, attribute_item, d) {
      // console.log('onPut', a, b, attribute_item, d)
      try {
        let type = attribute_item.attributes['display_type'].value;
        return type == 'CHECKBOX' || type == 'SWITCH';
      } catch (error) {
        return false;
      }
    },
    selectedActiveValue(attribute){
      if (isBoolean(attribute)){
        return isBooleanChecked(attribute);
      } else {
        var t;
        if(attribute.active_value) {
          t = Object.keys(attribute.options).find(l=>{
            return attribute.options[l] == attribute.active_value;
          })
          if(!t) {
            attribute.active_value = Object.values(attribute.options)[0];
          }
        }
        if(!t) t = Object.keys(attribute.options)[0];
      
        return attribute.value == attribute.options[t];
      }
    }
  },
  computed: {
    legendAttr(){
      return this.attribute.legends[0];
    },
    isLegendControlVisible(){
      return this.attribute.legend_control == 'Visible';
    },
    isLegendControlDisable(){
      return this.attribute.legend_control == 'Disable';
    },
    visibleItems(){
      return (this.legendAttr && this.selectedActiveValue(this.legendAttr));
    },
    disabledItems(){
      return !(this.legendAttr && this.selectedActiveValue(this.legendAttr));
    },
  },
  mounted() {
    // for (let i = 0; i < this.attribute.attributes.length; i++) {
    //   const a = this.attribute.attributes[i];
    //   a.path = this.attribute.path + '/' + this.attribute.key;
    // }
  },
  created: function(){
    // console.log('attribute create', this.attribute.access, this.key)

    if(!this.attribute.on_edit){
      if(!this.parent.path){
        this.parent.path = ""
      }
  
      this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`
    }

    console.log(this.attribute.path, this.parent)
  }
});

// Handle Attribute
Vue.component("resize-droparea", {
  model: {
  },
  props: {
    attribute: {
      type: Object,
    },
  },
  template: `
    <div class="resize-droparea">
    </div>
  `
});
