

// Attribute
Vue.component("attribute", {
  model: {
    prop: "attribute",
    event: "change",
    event: "input",
    event: "submit",
    event: "select",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    },
    group: {
      type: Object,
    },
    layout: {
      type: Object,
    },
    index: {
      type: Number,
    },
    disabled: {
      default: false,
      type: Boolean,
    }, 
    reverse: {
      default: false,
      type: Boolean,
    },
    labelWidth: {
      default: '',
      type: String,
    },
    onEdit: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
  },
  template: `
  <table :class="attributeClass">
    <tr class="attribute-input">
      <template v-if="attribute.access == 'Block'">
        <td>
          <block-attribute v-model="attribute"
          :ref="attribute.path"
          :parent="parent"
          :on-edit="onEdit"
          :disabled="disabled"
          :display_action="display_action">
          </block-attribute>
        </td>
      </template>
      <template v-else>
        <!-- handle -->
        <td>
          <handle-attribute v-if="onEdit && display_action"></handle-attribute>
        </td>

        <!-- Icon -->
        <td class="minimum-width">
          <attribute-icon v-model="attribute"></attribute-icon>
        </td>

        <!-- Item -->
        <td>
          <template v-if="$root.isCustomAttribute(attribute)">
            <template v-if="attribute.access == 'Image'">
              <image-attribute v-model="attribute"
              :parent="parent"
              :width="attribute.width"
              :height="attribute.height"
              :disabled="disabled"
              :on-edit="onEdit"
              :display_action="display_action"
              ></image-attribute>
            </template>
            <template v-else-if="attribute.access == 'Fieldset'">
              <fieldset-attribute v-model="attribute"
                :ref="attribute.path"
                :parent="parent"
                :disabled="disabled"
                :on-edit="onEdit"
                :display_action="display_action"
                >
                <span>content</span>
              </fieldset-attribute>
            </template>
            <template v-else>
              <divider-attribute v-model="attribute"
                :parent="parent"
                :disabled="disabled"
                :on-edit="onEdit"
                :display_action="display_action"
                ></divider-attribute>
            </template>
          </template>

          <template v-else>
            <el-form-item :label-width="computedLabelWidth">
              <!-- Label -->
              <template slot="label" v-if="(reverse == false || attribute.display_type == 'SWITCH')">
                <attribute-label v-model="attribute" :on-edit="onEdit" :path="attribute.path"></attribute-label>
              </template>
              
              <attribute-input
                v-model="attribute"
                @change="$emit('change', attribute)"
                @input="$emit('input', attribute)"
                @submit="$emit('submit')"
                @mouseover="$root.attributeMouseover(attribute)"
                @mouseleave="$root.attributeMouseleave(attribute)"
                v-bind:disabled="disabled"
                v-bind:reverse="reverse"
                v-bind:value="attribute.value"
                v-bind:display_value="attribute.display_value"
                v-bind:on-edit="onEdit"
              >
              </attribute-input>
            </el-form-item>
          </template>
        </td>

        <td class="action-right">
          <edit-attribute  v-if="display_action"
            v-model="attribute" :parent="parent" :on-edit="onEdit">
          </edit-attribute>
        </td>
      </template>
    </tr>
  </table>
  `,
  methods: {
    selectAttribute(){
      // remove all selected attributes
      clearSelectedAttribute();
      this.attribute.selected = true;
      this.$emit('select', this.attribute);
    },
    setPath(){
      // console.log('attribute.setPath', this.attribute.access, this.attribute.formlabel)
      if (!this.attribute.on_edit){
        // this.attribute.parent = this.parent;
        
        if(!this.parent.path){
          this.parent.path = ''
        }
    
        this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`

        if(this.attribute.access == 'Block'){
          for (let i = 0; i < this.attribute.attributes.length; i++) {
            const col = this.attribute.attributes[i];
            col.path = this.attribute.path + '/' + `${this.attribute.access}::${this.attribute.key}`;
          }
        }
      }
    }
  },
  watch: {
    attribute(n, o){
      // console.log('attribute change', n)
      this.setPath();
    }
  },
  computed: {
    computedLabelWidth() {
      if (this.reverse && this.attribute.display_type != 'SWITCH'){
        return '0px';
      }

      return this.labelWidth;
    },
    attributeClass(){
      var c = 'table-layout-attributes ' + this.attribute.access;
      if(this.onEdit) {c += ' on-edit'};
      if(this.attribute.on_edit) {c += ' attribute-on-edit'};
      c += ' attribute-is-' + (this.attribute.hide ? 'hide' : 'show')
      return c; 
    }
  },
  mounted: function (){
    // console.log('attribute mounted', this.attribute.formlabel);
    var self = this;

    this.attribute.onEdited = function(action) {
      // console.log('attribute.onEdited', action)
      self.$root.attributeChangedProp(self.attribute)
    }

    this.attribute.refreshLabel = function() {
      // console.log('refreshForm', self.labelWidth)
      self.labelWidth = '100px';
      self.$nextTick((_)=>{
        self.labelWidth = '';
      })
    }
    // console.log('attribute.mounted', 'set_path', this.attribute.path)
    // this.setPath();
  },
  created: function(){
    // console.log('attribute.created', this.attribute)
    this.setPath();
  }
});