// Attribute
Vue.component("block-attribute", {
  model: {
    prop: "attribute",
    event: "change",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    },
    disabled: {
      default: false,
      type: Boolean,
    },  
    height: {
      default: 20,
      type: Number,
    },  
    labelWidth: {
      default: 'auto',
      type: String,
    }, 
    reverse: {
      default: false,
      type: Boolean,
    }, 
    onEdit: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
  },
  template: `
  <div class="block-attribute" :class="onEdit ? 'on-edit' : ''">
    <el-card shadow="never" :style="style">
      <div slot="header" v-if="onEdit || hasTitle() "
        class="block-attribute-header">
        <div style="display: flex; align-items: center;">
          <handle-attribute v-if="onEdit && display_action"></handle-attribute>

          <!-- Icon -->
          <attribute-icon v-model="attribute"></attribute-icon>

          <!-- <span>Card name</span> -->
          <span @dblclick="editName()">
            <template v-if="attribute.name == ''">
              <span style="color: #909399">Block Title</span>
            </template>
            <template v-else-if="!attribute.hide_label">
              <span v-html="attribute.name" :style="labelStyle(attribute)"></span>
            </template>
          </span>
        </div>

        <template v-if=onEdit>
          <div>
            <el-select v-model="attribute.num_col" class="select-icons" style="width: 50px;"
              @change="changeNumberColumns" placeholder="" size="mini" :disabled="false">
              <template slot="prefix">
                <i v-for="i in attribute.num_col" :key="i" class="fa fa-square-o"></i>
              </template>
              <!-- <template slot="suffix">
                <i v-for="i in attribute.num_col" :key="i" class="fa fa-square-o"></i>
              </template> -->
    
              <el-option :value="1" :label="' '"> <i class="fa fa-square-o"></i> </el-option>
              <el-option :value="2" :label="' '"> <i class="fa fa-square-o"></i> <i class="fa fa-square-o"></i> </el-option>
            </el-select>
            <edit-attribute v-model="attribute" :parent="parent" :on-edit="onEdit" :inline="true">
            </edit-attribute
          </div>
        </template>
      </div>

      <el-row :gutter="10" style="display: flex;">
        <template v-for="(column, index) in attribute.attributes">
          <el-col :span="colWidth(column)" :key="index"
            :ref="column.key"
            :style="columnWidth(column, index)"
          >
            <div style="display: flex; align-items: center;">
              <!-- Icon -->
              <attribute-icon v-model="column"></attribute-icon>

              <template v-if="column.name != '' && !column.hide_label" >
                <span v-html="column.name" :style="labelStyle(column)"></span>
              </template>
            </div>

            <draggable tag="el-form" handle=".handle-attribute"
              :list="column.attributes" 
              :component-data="getFormData(column)"
              @add="changeItems(column)"
              @remove="changeItems(column)"
              @update="changeItems(column)"
              @submit.prevent.native=""
              v-bind='{group: "layout.attributes"}'>
              
              <div v-for="(a, attr_index) in column.attributes" :key="a.name || a.key" 
              :class="'attribute-item ' + a.access + (onEdit ? ' on-edit' : '')"
              :display_type="a.display_type"
              :style="$root.attributeItemStyle" 
                v-if="$root.attributeVisible(a) && (a.render || a.render == undefined)"
                >
    
                <attribute
                  v-model="a"
                  :parent="column"
                  :on-edit="onEdit"
                  :display_action="display_action"
                  :disabled="disabled ? disabled : $root.attributeDisabled(a)"
                  @change="$root.changeAttrValue"
                  @input="$root.onChangeAttrValue(a)"
                  @submit="$root.submit"
                ></attribute>
              </div>
    
              <el-form-item v-if="column.attributes.length == 0" label="">
                <div style="height: 32px; padding: 0 5px; border: 1px dashed gray;">No Attribute</div>
              </el-form-item>
  
              <div v-if="onEdit" slot="footer">
                <el-form size="mini" :disabled="false" >
                  <el-button @click="$root.editLayout(column, refSize(column.key))"
                    size='mini' class="icon-button">
                    <i class="fa fa-sliders fa-2x"></i>
                  </el-button>
                </el-form>
              </div>
            </draggable>
          </el-col>
          <template v-if="attribute.attributes.length == 2 && index == 0">
            <el-divider direction="vertical"></el-divider>
          </template>
        </template>
      </el-row>
      <div v-if="showSliderSize()"> 
        <el-slider v-model="attribute.attributes[0].span" @input="changeSpan($event)"
          :min="0" :max="24" :step="1" show-stops :show-tooltip="false"></el-slider>
      </div>
    </el-card>
  </div>
  `,
  methods: {
    hasTitle(){
      if (this.attribute.hide_label) {
        if (this.attribute.image && this.attribute.access != 'Image'){
          return true;
        } else {
          return false;
        }
      } else if (this.attribute.name == ''){
        return false;
      }

      return true;
    },
    getFormData(column){
      return {
        attrs:{
          'disabled': this.disabled,
          'label-width': this.labelWidth,
          'label-position': column.labelposition,
          'size': 'mini',
          'class': 'form-layout',
        },
      };
    },
    editName(){
      if (!this.onEdit) return;

      this.$prompt('Enter title', 'Block Title', {
        inputValue: this.attribute.name,
        inputPlaceholder: 'Title',
        confirmButtonText: 'OK',
        cancelButtonText: 'Cancel',
      }).then(({ value }) => {
        this.attribute.name = value;
      }).catch(() => {
      });
    },
    refSize(ref){
      return this.$refs[ref][0];
    },
    changeItems(column){
    },   
    showSliderSize(){
      return this.onEdit && 
        this.attribute.attributes.length == 2 && 
        !this.attribute.attributes.find(c=>{ return c.fixed_width });
    },   
    columnWidth(column, index){
      // Nếu 1 trong 2 fixed_width thì col còn lại là flex_grow = 1
      if(this.attribute.attributes.length == 1){
        return {};
      }

      var style = {};

      if(column.fixed_width){
        style.width = column.width + 'px';
        style['flex-shrink'] = 0;
      } else {
        let otherColumn = this.otherColumn(column);
        // console.log(otherColumn)
        if(otherColumn && otherColumn.fixed_width){
          style['flex-grow'] = 1;
        } else {
          // Use span prop
          // return {};
        }
      }

      if (column.use_min_width) style['min-width'] = column.min_width + 'px';
      if (column.use_max_width) style['max-width'] = column.max_width + 'px';
      return style;
    },
    otherColumn(column){
      return this.attribute.attributes.find(c=>{ return c.key != column.key });
    },
    changeNumberColumns() {
      if(this.attribute.num_col == 1){
        col2 = this.attribute.attributes[1]
        if(col2){
          for (let i = 0; i < col2.attributes.length; i++) {
            const a = col2.attributes[i];
            this.attribute.attributes[0].attributes.push(a)
          }
          this.$delete(this.attribute.attributes, 1);
        }
      } else {
        this.attribute.attributes.push(
          {
            render: true,
            access: 'Column',
            hide_label: false,
            label_style: [],
            name: '',
            key: parse_uuid(),
            path: this.attribute.path + '/' + `${this.attribute.access}::${this.attribute.key}`,
            span: 12,
            attributes: []
          }
        )
      }

      this.$forceUpdate()
    },
    changeSpan(span) {
      if(this.attribute.num_col == 1){
      } else {
        // console.log(span)
        if(this.attribute.attributes[1]){
          this.attribute.attributes[1].span = 24 - span
        }
      }
    },
    colWidth(column) {
      return this.attribute.attributes.length == 1 ? 24 : (column.span == 0 ? 24 : column.span);
    },
    labelStyle(a) {
      return {
        'font-weight': a.label_style.includes('bold') ? 'bold' : 'normal',
        'font-style': a.label_style.includes('italic') ? 'italic' : 'normal',
        'text-decoration': a.label_style.includes('underline') ? 'underline' : 'unset',
        color: a.label_text_color
      }
    }
  },
  watch: {
  },
  computed: {
    style() {
      return {
        'background-color': this.attribute.background_color, 
        border: this.onEdit ? '' : (this.attribute.border ? '' : 'none'),
        margin: this.attribute.border ? '0 0 5px 0' : '0'
      }
    },
  },
  mounted: function (){
    var self = this;
    this.attribute.vaildSize = function(path) {
      return path;
    }

    this.attribute.refreshForm = function() {
      // console.log('refreshForm', self.labelWidth)
      self.labelWidth = '100px';
      self.$nextTick((_)=>{
        self.labelWidth = 'auto';
      })
    }
  },
  created: function(){
    // if (!this.attribute.on_edit){
    //   if (!this.parent.path) {
    //     this.parent.path = ""
    //   }
  
    //   this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`
  
    //   for (let i = 0; i < this.attribute.attributes.length; i++) {
    //     const col = this.attribute.attributes[i];
    //     col.path = this.attribute.path + '/' + `${this.attribute.access}::${this.attribute.key}`;
    //   }
    // }
  }
});