// Display Types
Vue.component("display-type", {
  model: {
    prop: "attribute",
    event: "change",
  },
  props: {
    attribute: {
      type: Object,
    },

    type: {
      type: String,
    },
   
    default_types: {
      default: true,
      type: Boolean,
    },
  },
  data: function () {
    return {
      display_type: [
        {
          label: "Default",
          value: "DEFAULT",
        },
        {
          label: "Materials",
          value: "MATERIAL",
        },
        {
          label: "Checkbox",
          value: "CHECKBOX",
        },
        {
          label: "Switch",
          value: "SWITCH",
        },
        {
          label: "Radio",
          value: "RADIO",
        },
        {
          label: "Slider",
          value: "SLIDER",
        },
      ],
      
    }
  },
  template: `
  <span>
    <el-select v-model="attribute.display_type"
      placeholder="Default"
      @change="$emit('change', attribute)">

      <el-option v-for="type in display_type" :label="type.label"
        style="height: 28px; line-height: 28px;"
        :value="type.value"
        :disabled="!(validDisplayType(attribute, type.value))">

        <span style="float: left">{{ type.label }}</span>

        <span style="float: right; width: 40%;">
          <template v-if="type.label == 'Default'">
            <span>{{attribute.access}}</span>
            <!-- <template v-if="attribute.access == 'TEXTBOX'">
              <el-input size="mini"></el-input>
            </template>
            <template v-else-if="attribute.access == 'LIST'">
              <el-select size="mini"></el-select>
            </template>
            <template v-else><span>{{attribute.access}}</span></template> -->
          </template>
          <template v-else-if="type.label == 'Materials'">
            <img src="../css/images/texture.jpg"
              style="height: 22px; width:22px;"
              class="thumbnail-inlist">
          </template>
          <template v-else-if="type.label == 'Checkbox'">
            <el-checkbox v-model="default_types">
            </el-checkbox>
          </template>
          <template v-else-if="type.label == 'Switch'">
            <el-switch width="40" v-model="default_types">
            </el-switch>
          </template>
          <template v-else-if="type.label == 'Radio'">
            <el-radio v-model="default_types" :label="true">A
            </el-radio>
            <el-radio v-model="default_types" :label="false">B
            </el-radio>
          </template>
          <template v-else-if="type.label == 'Slider'">
            <el-slider :value="50"></el-slider>
          </template>
        </span>
      </el-option>
    </el-select>

    <slot></slot>

  </span>
  `,
  methods: {
    validDisplayType(attribute, type) {
      // console.log(attribute, type);
      if (type == "SLIDER") {
        if (
          attribute.units == "INTEGER" ||
          attribute.units == "FLOAT" ||
          attribute.units == "DEGREES" ||
          attribute.units == "PERCENT"
        ) {
          return true;
        } else {
          return false;
        }
      } else if (type == "CHECKBOX") {
        if (attribute.access != "LIST") {
          return false;
        }
        return isBoolean(attribute);
      } else if (type == "SWITCH") {
        if (attribute.access != "LIST") {
          return false;
        }
        return isSwitch(attribute);
      } else if (type == "MATERIAL") {
        if (attribute.access != "LIST") {
          return false;
        }
        if (isBoolean(attribute)) {
          return false;
        }
      } else if (type == "RADIO") {
        return attribute.access == "LIST";
      }
      return true;
    },
  },
  watch: {
    type: function(newVal, oldVal){
      // if (oldVal == undefined) return;

      console.log('type changed: ', oldVal , ' => ', newVal)

      if (oldVal == 'MATERIAL'){
        if (this.attribute.image_options == undefined){
          this.attribute.option_use_image = false;
        }
      }

      if (newVal == "SLIDER") {
        if (this.attribute.slidermin == undefined) {
          this.attribute.slidermin = 0;
          this.attribute.slidermax = 100;
          this.attribute.sliderstep = 5;
        }
      } else if (newVal == "RADIO") {
        if (this.attribute.radiostyle == undefined) {
          this.attribute.radiostyle = "DEFAULT";
        }

        this.$root.setDefaultRadioImageData(this.attribute)
      } else if (newVal == "SWITCH"){
        if (this.attribute.active_value == undefined){
          if (isBoolean(this.attribute)){
            this.attribute.active_value = '0'
          }else {
            this.attribute.active_value = Object.values(this.attributes.options[0])
          }
        }
      } else if (newVal == 'MATERIAL'){
        this.attribute.option_use_image = true;

        if (this.attribute.option_image_size == undefined){
          this.attribute.option_image_size = 'small';
        }
      }
    },
  },
  mounted() {
    // this.type = this.attribute.display_type;
    console.log(this.type)
  },
  created(){
    
  }
});