// Fieldset
Vue.component("fieldset-attribute", {
  model: {
    prop: "attribute",
    event: "change-attributes",
    event: "change-legend",
  },
  props: {
    attribute: {
      type: Object,
    },
    parent: {
      type: Object,
    },
    group: {
      type: Object,
    },
    layout: {
      type: Object,
    },
    index: {
      type: Number,
    },
    labelWidth: {
      default: 'auto',
      type: String,
    },
    onEdit: {
      default: false,
      type: Boolean,
    }, 
    disabled: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
  },
  data: function () {
    return {
    };
  },
  template: `
  <span>
    <el-card shadow="never" :style="cardStyle">
      <!-- <Legend></Legend> -->
      <div slot="header">
        <draggable tag="el-form" handle=".handle-attribute"
          :list="attribute.legends" 
          @remove="changeLegend"
          @add="changeLegend"
          @update="changeLegend"
          @submit.prevent.native=""
          :component-data="{
              attrs:{
                'disabled': disabled,
                'label-width': 'auto',
                'size': 'mini',
                'class': 'form-layout',
              }
            }"
          v-bind='{group: {
            name: "layout.attributes",
            pull: true,
            put(a, b, c, d) {
              // Return true to allow, return false to disallow
              // return true;
              return onPut(a, b, c, d);
            }
          }}'>
  
          <div v-for="(a, attr_index) in attribute.legends" :key="attribute.name || attribute.key" 
            :class="'attribute-item ' + a.access + (onEdit ? ' on-edit' : '')"
            :display_type="attribute.display_type"
            :style="$root.attributeItemStyle">
            <attribute
              v-model="a"
              :parent="attribute"
              :on-edit="onEdit"
              :display_action="display_action"
              reverse="true"
              @change="$root.changeAttrValue"
            ></attribute>
          </div>
  
          <el-form-item v-if="attribute.legends && attribute.legends.length == 0" label="">
            <div style="border: 1px dashed darkgrey; height: 32px; padding: 0 5px;" title="Only Checkbox Attribute!">No Legend</div>
          </el-form-item>
        </draggable>
      </div>
      
      <!-- Items -->
      <div v-if="(isLegendControlVisible && !onEdit) ? visibleItems : true">
        <!-- <el-divider></el-divider> -->
        <draggable tag="el-form" handle=".handle-attribute"
          :list="attribute.attributes"
          @submit.prevent.native=""
          :component-data="getFormData()"
          v-bind='{group: "layout.attributes"}'>
          <div v-for="(a, attr_index) in attribute.attributes" :key="a.name || a.key" class="attribute-item" :style="$root.attributeItemStyle">

            <template v-if="(isLegendControlVisible && !onEdit) ? visibleItems : true">
              <attribute
                v-model="a"
                :parent="attribute"
                :on-edit="onEdit"
                :display_action="display_action"
                :disabled="disabled ? disabled : (isLegendControlDisable ? disabledItems : false)"
                @submit="$root.submit"
                @change="$root.changeAttrValue"
                @input="$root.onChangeAttrValue(a)"
              ></attribute>
            </template>

          </div>
          <el-form-item v-if="attribute.attributes.length == 0" label="">
            <div style="border: 1px dashed darkgrey; height: 32px; padding: 0 5px;">No Attribute</div>
          </el-form-item>
        </draggable>
      </div>
  </el-card>
  </span>

  `,
  methods: {
    getFormData(){
      return {
        attrs:{
          'disabled': this.disabled ? 
                      this.disabled : 
                      (this.isLegendControlDisable ? 
                                this.disabledItems : 
                                false
                      ),
          'label-width': this.labelWidth,
          'size': 'mini',
          'class': 'form-layout',
        }
      };
    },
    addFieldsetLegend() {
      this.$prompt('Select attribute', 'Legend', {
        confirmButtonText: 'OK',
        cancelButtonText: 'Cancel',
        inputPlaceholder: 'Enter attribute name (Only Check or Switch Type)',
        inputErrorMessage: 'Invalid attribute',
        inputValidator(value){
          let a = this.$root.dcAttribute(value)
          checktype = a && (this.$root.isSwitch(a.options) || this.$root.isBoolean(a.options));
          // console.log(a)
          return (a && checktype) ? true : false;
        }
        // inputPattern: /[\w!#$%&'*+/=?^_`{|}~-]+(?:\.[\w!#$%&'*+/=?^_`{|}~-]+)*@(?:[\w](?:[\w-]*[\w])?\.)+[\w](?:[\w-]*[\w])?/,
        // inputErrorMessage: 'Invalid Email'
      }).then(({ value }) => {
        this.attribute.legend = this.$root.dcAttribute(value);


        this.$message({
          type: 'success',
          message: 'Attribute:' + value
        });
      }).catch(() => {
        this.$message({
          type: 'info',
          message: 'Input canceled'
        });       
      });
    },
    changeLegend(e){
      // console.log('changeLegend', e)
      return false;
      // console.log(this.$root)
      legend = this.attribute.legends[this.attribute.legends.length-1]
      // console.log(legend)

      if(this.attribute.legends.length == 2){
        this.$alert('Legend must be Checkbox.', 'Invalid Type!', {
          confirmButtonText: 'OK',
          callback: action => {
            this.$delete(this.attribute.legends, this.attribute.legends.indexOf(legend))
          }
        });
      } else if(legend){
        if(isBoolean(legend)){
        }else{
          this.$alert('Legend must be Checkbox.', 'Invalid Type!', {
            confirmButtonText: 'OK',
            callback: action => {
              this.$delete(this.attribute.legends, this.attribute.legends.indexOf(legend))
            }
          });
        }
      }
    },
    onPut: function(a, b, attribute_item, d) {
      // console.log('onPut', a, b, attribute_item, d)
      try {
        let type = attribute_item.attributes['display_type'].value;
        return type == 'CHECKBOX' || type == 'SWITCH';
      } catch (error) {
        return false;
      }
    },
    selectedActiveValue(attribute){
      if (isBoolean(attribute)){
        return isBooleanChecked(attribute);
      } else {
        var t;
        if(attribute.active_value) {
          t = Object.keys(attribute.options).find(l=>{
            return attribute.options[l] == attribute.active_value;
          })
          if(!t) {
            attribute.active_value = Object.values(attribute.options)[0];
          }
        }
        if(!t) t = Object.keys(attribute.options)[0];
      
        return attribute.value == attribute.options[t];
      }
    }
  },
  computed: {
    cardStyle(){
      return {
        'background-color': this.attribute.background_color,
        border: this.onEdit ? '' : (this.attribute.border ? '' : 'none'),
        margin: this.attribute.border ? '0 0 5px 0' : '0'
      }
    },
    legendAttr(){
      return this.attribute.legends[0];
    },
    isLegendControlVisible(){
      return this.attribute.legend_control == 'Visible';
    },
    isLegendControlDisable(){
      return this.attribute.legend_control == 'Disable';
    },
    visibleItems(){
      return (this.legendAttr && this.selectedActiveValue(this.legendAttr));
    },
    disabledItems(){
      return !(this.legendAttr && this.selectedActiveValue(this.legendAttr));
    },
  },
  mounted() {
    this.attribute.refreshForm = function() {
      // console.log('refreshForm', self.labelWidth)
      self.labelWidth = '100px';
      self.$nextTick((_)=>{
        self.labelWidth = 'auto';
      })
    }
  },
  created: function(){
    // console.log('fieldset-attribute create', this.attribute.access, this.key)

    if(!this.attribute.on_edit){
      if(!this.parent.path){
        this.parent.path = ""
      }
  
      this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`
    }

    // console.log(this.attribute.path, this.parent)
  }
});