// Image Attribute
Vue.component("image-attribute", {
  model: {
    prop: "attribute",
    event: "change",
  },
  props: {
    attribute: {
      default: {},
      type: Object,
    },
    parent: {
      type: Object,
    },
    width: {
      type: Number,
    },
    height: {
      type: Number,
    },
    onEdit: {
      default: false,
      type: Boolean,
    }, 
    disabled: {
      default: false,
      type: Boolean,
    },
    display_action: {
      default: true,
      type: Boolean,
    },
    grid: {
      default: [1, 1],
      type: Array,
    },
    position: {
      default: '',
      type: String,
    }, 
    update_pos: {
      default: false,
      type: Boolean,
    },
    zindex: {
      default: 100,
      type: Number,
    },
  },
  data: function () {
    return {
    }
  },
  template: `
    <div @dblclick="insertField($event)" v-on:keyup.up="moveUp($event)" style="position: relative;">
      <template v-if="!attribute.on_edit">
        <vue-draggable-resizable v-for="(inputGroup, index) in attribute.attributes" :key="inputGroup.key"
          :w="inputGroup.w" :h="inputGroup.h"
          :x="inputGroup.x" :y="inputGroup.y"
          :min-width="50"
          :ref="inputGroup.key"
          :z="100"
          :handles="['mr','ml', 'bm']"
          :parent="true" 
          :grid=grid
          :drag-handle="'.drag-handle'" 
          class-name="image-input-group"
          class-name-active="image-input-active"
          :style="draggableResizableStyle"
          @activated="onActivated(inputGroup)" @deactivated="onDeactivated(inputGroup)"
          @dragging="onDrag" @resizing="onResize" 
          @resizestop="(x, y, width, height) => {onResizeStop(x, y, width, height, inputGroup)}"
          @dragstop="(x, y) => {onDragStop(x, y, inputGroup)}"
          :enable-native-drag="true">
  
          <div class="image-input-action" v-if="onEdit || attribute.on_edit">
            <span class="drag-handle action-icon">
              <i class="fa fa-thumb-tack" aria-hidden="true"></i>
            </span>
  
            <span class="action-icon"
              @click="removeInputGroup(inputGroup, index)"
              style="float: right;">
              <i class="fa fa-trash" aria-hidden="true"></i>
            </span>
          </div>
  
          <draggable tag="el-form" handle=".handle-attribute"
            :list="inputGroup.attributes" 
            @add="changeInputs(inputGroup)"
            @remove="changeInputs(inputGroup)"
            @update="changeInputs(inputGroup)"
            @submit.prevent.native=""
            :component-data="{
                attrs:{
                  'disabled': disabled,
                  'label-width': 'auto',
                  'size': 'mini',
                  'class': 'form-layout',
                }
              }"
            v-bind='{group: "layout.attributes"}'>
  
            <div v-for="(a, attr_index) in inputGroup.attributes" :key="a.name || a.key" 
            :class="'attribute-item ' + a.access + (onEdit ? ' on-edit' : '') + ' image-float-attribute-item'"
            :style="$root.attributeItemStyle">
              <attribute
                v-model="a"
                :disabled="disabled ? disabled : $root.attributeDisabled(a)"
                :parent="inputGroup"
                :on-edit="onEdit"
                :display_action="display_action"
                @submit="$root.submit"
                @change="$root.changeAttrValue"
                @input="$root.onChangeAttrValue(a)"
              ></attribute>
            </div>
  
            <el-form-item v-if="inputGroup.attributes.length == 0" label="">
              <div style="border: 1px dashed darkgrey; height: 32px; padding: 0 5px;">No Attribute</div>
            </el-form-item>
          </draggable>
        </vue-draggable-resizable>
      </template>

      <div :style="{width: '100%', display: 'flex', 'justify-content': attribute.position, 'z-index': 1, 'user-select': 'none'}">
        <el-card :style="cardStyle" :body-style="{padding: 0, display: 'flex'}" shadow="never">
          <el-image
            @load="onLoadImage($event)"
            :style="imageSize"
            :src="attribute.image.base64" fit="contain">
          </el-image>
        </el-card>
      </div>
    </div>
  `,
  methods: {
    onLoadImage (e) {
      if (this.attribute.image.naturalWidth) return;

      this.attribute.image.naturalWidth = e.target.naturalWidth;
      this.attribute.image.naturalHeight = e.target.naturalHeight;

      if (!this.attribute.width){
        if(this.attribute.image.naturalWidth > 500){
          this.attribute.width = 250;
        } else {
          this.attribute.width = this.attribute.image.naturalWidth;
          this.attribute.height = this.attribute.image.naturalHeight;
        }
      }
    },
    moveUp (e) {
      // console.log(e)
      e.stopPropagation()
    },
    onActivated (inputGroup) {
      inputGroup.active = true;
      // console.log(inputGroup.active)
    }, 
    onDeactivated (inputGroup) {
      inputGroup.active = false;
      // console.log(inputGroup.active)
    },
    insertField(e){
      // console.log(e);
      if(!this.onEdit){
        return
      }

      block = {
        access: 'FloatInput',
        key: parse_uuid(),
        x: e.offsetX,
        y: e.offsetY,
        w: 150,
        h: 'auto',
        attributes: []
      }

      this.attribute.attributes.push(block)
    },

    removeInputGroup(inputGroup, index){
      // move floatinput to parent of image attribute
      this.$root.removeCustomAttribute(inputGroup, this.parent);

      // remove floatinput from image attributes
      this.$delete(this.attribute.attributes, index);
    },

    changeInputs(inputGroup){
      // console.log('changeInputs', inputGroup)
      this.updateBorderHeight(inputGroup)
    },

    imagePosition() {
      c_img = this.$children.find(c=>{ return c.$options._componentTag == 'el-card'; })
      let position = [c_img.$el.offsetLeft, c_img.$el.offsetTop]
      return position
    },

    onResize: function (x, y, width, height) {
      // console.log(x, y, width, height)
      this.grid = [5, 5];
    }, 
    onResizeStop: function (x, y, width, height, inputGroup) {
      inputGroup.x = x;
      inputGroup.y = y;
      inputGroup.w = width;
      inputGroup.h = this.updateBorderHeight(inputGroup);

      this.grid = [1, 1];

      this.pinBlockPosition(inputGroup)
    },
    onDrag: function (x, y) {
      // console.log(this)
      // console.log(x, y)
    },
    onDragStop: function (x, y, inputGroup) {
      // console.log('onDragStop')
      inputGroup.x = x;
      inputGroup.y = y;
      this.pinBlockPosition(inputGroup)
    },
  
    pinBlockPosition: function (block) {
      // console.log('pinBlockPosition()', [block.x, block.y], this.imagePosition())
      block.pin_position = this.imagePosition();
    },

    updateBorderHeight(inputGroup){
      let self = this.$refs[inputGroup.key][0]
      self.$el.style.height = 'auto'
      
      var style = window.getComputedStyle(self.$el);
      height = parseFloat(style.getPropertyValue('height'), 10)
      
      self.height = height;
      s = self.checkParentSize();

      return height;
    },

    updatePosition: function () {
      if(this.update_pos == false){
        return;
      }
      this.update_pos = false

      const img_pos = this.imagePosition()

      if(this.attribute.on_edit){
        return;
      }
      for (let i = 0; i < this.attribute.attributes.length; i++) {
        const block = this.attribute.attributes[i];
        const move_x = img_pos[0] - block.pin_position[0]
        if(move_x != 0){
          block.x += move_x;
          this.pinBlockPosition(block);
        }
      }
    },
  },
  watch: { 
    attribute: function(newVal, oldVal){
      this.update_pos = true;
      this.updatePosition();
    },
    onEdit: function(newVal, oldVal){
      // console.log('onEdit changed: ', newVal)
      this.update_pos = true;
    },
    on_edit: function(newVal, oldVal){
      // console.log('onEdit changed: ', newVal)
      this.update_pos = true;
    }, 
    position: function(newVal, oldVal){
      // console.log('position changed: ', newVal)
    },
    display_action: function(newVal, oldVal){
      // console.log('display_action changed: ', newVal)
      this.update_pos = true;
    },
    width: function(newVal, oldVal){
      this.attribute.height = Math.round((newVal/this.attribute.image.naturalWidth) * this.attribute.image.naturalHeight);
    },
    height: function(newVal, oldVal){
      this.attribute.width = Math.round((newVal/this.attribute.image.naturalHeight) * this.attribute.image.naturalWidth);
    },
  },
  computed: {
    imageSize(){
      return {
        width: this.attribute.width + 'px',
        height: this.attribute.height + 'px',
        // height: (this.attribute.width/this.attribute.image.naturalWidth) * this.attribute.image.naturalHeight + 'px',
      }
    },
    cardStyle(){
      return {
        padding: 0,
        'background-color': this.attribute.background_color,
        border: this.onEdit ? '' : (this.attribute.border ? '' : 'none'),
      }
    },
    draggableResizableStyle(){
      if(this.onEdit || this.attribute.on_edit){
        return {
          // border: '1px solid darkgrey',
          background: 'rgba(245, 245, 245, 0.59)'
        }
      } else{
        return {
          // border: 'rgba(245, 245, 245, 0.59)',
          background: 'none'
        }
      }
    },
  },
  updated: function (){
    this.$nextTick(function () {
      // Code that will run only after the
      // entire view has been re-rendered
      this.updatePosition()
    })
  },
  mounted: function (){
    if(!this.attribute.on_edit){
      if(!this.parent.path){
        this.parent.path = ""
      }
  
      this.attribute.path = this.parent.path + '/' + `${this.parent.access}::${this.parent.key}`
  
      for (let i = 0; i < this.attribute.attributes.length; i++) {
        const a = this.attribute.attributes[i];
        a.path = this.attribute.path + '/' + `${this.attribute.access}::${this.attribute.key}`;
      }
    }
    var self = this;
    this.attribute.onEdited = function(e) {
      self.update_pos = true;
      self.updatePosition()
    }
    this.position = this.attribute.position;
    this.update_pos = true;
    this.updatePosition()

    this.$nextTick(() => {
      window.addEventListener('resize', () => {
        this.update_pos = true;
        this.updatePosition()
      });

      window.addEventListener('keydown', (event) => {
        if (![37, 38, 39, 40].includes(event.keyCode)){
          return;
        }

        activeBlocks = this.attribute.attributes.filter(block=>{
          return block.active;
        })

        if(activeBlocks.length == 0){
          return;
        } else {
          event.stopPropagation();
        }

        if (event.keyCode == 37) {
          // left = 37
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.x -= this.grid[0]
          }
        } 
        else if (event.keyCode == 38){
          // up = 38
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.y -= this.grid[1]
          }
        }
        else if (event.keyCode == 39){
          // right = 39
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.x += this.grid[0]
          }
        }
        else if (event.keyCode == 40){
          // down = 40
          for (let i = 0; i < activeBlocks.length; i++) {
            const b = activeBlocks[i];
            b.y += this.grid[1]
          }
        }
      });
    })
  }
});
