// Input
Vue.component("attribute-input", {
  model: {
    prop: "attribute",
    event: "focus",
    event: "change",
  },
  props: {
    attribute: {
      default: {},
      type: Object,
    },
    suggestions: {
      type: Array,
    },
    value: {
      type: [String, Number, Boolean, Object],
    },
    display_value: {
      type: [String, Number, Boolean, Object],
    },
    onCancel: {
      default: false,
      type: Boolean,
    },
    cb_suggestion: {
      type: Object,
    },
    disabled: {
      type: Boolean,
    },
    reverse: {
      default: false,
      type: Boolean,
    }, 
    onEdit: {
      default: false,
      type: Boolean,
    },
  },
  data: function () {
    return {
      optionImageSizes: {
        tiny: {
          padding: "2px 5px !important",
          imageSize: {
            height: "28px",
            "max-width": "32px",
          },
        },
        mini: {
          padding: "2px 5px !important",
          imageSize: {
            height: "36px",
            "max-width": "48px",
          },
        },
        small: {
          padding: "5px 7px !important",
          imageSize: {
            height: "52px",
            "max-width": "64px",
          },
        },
        medium: {
          padding: "5px 10px !important",
          imageSize: {
            height: "68px",
            "max-width": "72px",
          },
        },
        large: {
          padding: "7px 15px !important",
          imageSize: {
            height: "76px",
            "max-width": "96px",
          },
        },
      },
      optionImageSize () {
        if(this.attribute.option_image_size){
          return this.optionImageSizes[this.attribute.option_image_size].imageSize;
        } else {
          return this.optionImageSizes['small'].imageSize;
        } 
      }
    };
  },
  template: `
  <span class="attribute-input">
    <table>
      <tr>
        <td>
          <template>
            <el-tooltip :ref="attribute.name + 'Tooltip'"
              :open-delay="$root.dialog_settings.tooltip_open_delay * 1000"
              effect="light" placement="top-end"
              :disabled="!attribute.tooltip">

              <template slot="content">
                <div :style="{width: attribute.tooltip ? attribute.tooltip.width + 'px' : 'auto', height: 'auto'}">
                  <div v-if="attribute.tooltip" v-html="attribute.tooltip.code"></div>
                </div>
              </template>

              <!-- Hover event contain -->
              <span :style="{width: (attribute.set_input_width ? attribute.input_width + 'px' : ''), display: 'block'}"
                @mouseover="$emit('mouseover', $event)"
                @mouseleave="$emit('mouseleave', $event)"
              >

                <template v-if="attribute.access == 'TEXTBOX'">
                  <template v-if="attribute.display_type == 'SLIDER'">
                    <el-slider v-model="attribute.display_value"
                      @change="$emit('change', attribute, true)"
                      @input="$emit('input', attribute, true)"
                      :disabled="disabled"
                      :min="attribute.slidermin" :max="attribute.slidermax"
                      :step="attribute.sliderstep"
                      :format-tooltip="$root.convertDisplayValue(attribute, attribute.display_value)">
                    </el-slider>
                  </template>
              
                  <template
                    v-else-if="attribute.units == 'INTEGER' || attribute.units == 'FLOAT'">
                    <el-input-number v-model="attribute.display_value"
                      controls-position="right"
                      @change="$emit('change', attribute)"
                      :disabled="disabled"
                      @keyup.enter.native="$emit('submit')"></el-input-number>
                  </template>
              
                  <template v-else-if="attribute.suggestions">
                    <el-autocomplete v-model="attribute.display_value"
                      :ref="'input_' + attribute.key"
                      :id="attribute.path + '/' + attribute.key"
                      :fetch-suggestions="querySuggestions"
                      @change="$emit('change', attribute)"
                      :disabled="disabled"
                      @select="$emit('change', attribute)"
                      @keyup.enter.native="$emit('submit')" size="mini">

                      <template slot-scope="{ item }">
                        <div class="value">{{ item.display_value }}</div>
                        <!-- <span class="link">{{ item.value }}</span> -->
                      </template>
                    </el-autocomplete>
                  </template>

                  <template v-else>
                    <el-input v-model="attribute.display_value"
                      :ref="'input_' + attribute.key"
                      :id="attribute.path + '/' + attribute.key"
                      @change="$emit('change', attribute)"
                      :disabled="disabled"
                      @keyup.enter.native="$emit('submit')" size="mini">
                    </el-input>
                  </template>
                </template>
              
                <template v-else-if="attribute.access == 'VIEW'">
                  <el-input v-model="attribute.display_value" readonly class="attribute-readonly">
                  </el-input>
                </template>
              
                <!-- List -->
                <template v-else-if="attribute.access == 'LIST'">
                  <template v-if="attribute.display_type == 'CHECKBOX'">
                    <el-checkbox v-model="attribute.value" 
                      true-label="0"
                      false-label="1" 
                      @change="$emit('change', attribute)"
                      :disabled="disabled">
                      <template v-if="reverse">
                        {{attribute.formlabel}}
                      </template>
                    </el-checkbox>
                  </template>
              
                  <!-- Switch -->
                  <template v-else-if="attribute.display_type == 'SWITCH'">
                    <el-switch v-model="attribute.value"
                      :active-value="activeValue()"
                      :inactive-value="inactiveValue()"
                      :active-text="attribute.option_hide_text ? '' : activeText()"
                      :inactive-text="attribute.option_hide_text ? '' : inactiveText()"
                      :title="attribute.option_hide_text ? inactiveText() + '/' + activeText() : ''"
                      inactive-color="#c8c8c8"
                      :disabled="disabled"
                      @change="$emit('change', attribute)"
                      :style="{float: reverse ? 'right' : ''}"
                      >
                    </el-switch>
                  </template>
              
                  <!-- Radio -->
                  <template v-else-if="attribute.display_type == 'RADIO'">
                    <el-radio-group v-model="attribute.value" size="mini"
                      :ref="'input_' + attribute.key"
                      :id="attribute.path + '/' + attribute.key"
                      :class="attribute.radiostyle == 'Image' ? 'image-radio' : ''"
                      :disabled="disabled"
                      :style="radioGroupStyle"
                      @change="$emit('change', attribute)">
                      <template v-if="!attribute.radiostyle || attribute.radiostyle == 'DEFAULT'">
                        <el-radio v-for="(v, n) in attribute.options"
                          :label="v">{{n}}</el-radio>
                      </template>
                      <template v-else-if="attribute.radiostyle == 'BORDER'">
                        <el-radio v-for="(v, n) in attribute.options" border
                          :label="v">{{n}}</el-radio>
                      </template>
                      <template v-else-if="attribute.radiostyle == 'BUTTON'">
                        <el-radio-button v-for="(v, n) in attribute.options"
                          :label="v">{{n}}</el-radio-button>
                      </template>
                      <template v-else-if="attribute.radiostyle == 'Image'">
                        <!-- <slot name="radio-image"></slot> -->
                        <el-radio v-for="(v, n) in attribute.options" :label="v" :key="n"
                          :style="{
                            padding: attribute.option_image_size ? optionImageSizes[attribute.option_image_size].padding : optionImageSizes['small'].padding
                          }"
                          border>
            
                          <el-image
                            :src="(attribute.image_options && attribute.image_options[v]) ? attribute.image_options[v].base64 : ''"
                            :atl="n" fit="scale-down"
                            @dblclick="onEdit ? $root.addOptionImage(attribute, v) : console.log(onEdit)"
                            :style="attribute.option_image_size ? optionImageSizes[attribute.option_image_size].imageSize : optionImageSizes['small'].imageSize">
            
                            <div slot="error"><i class="fa fa-image fa-2x"></i></div>
                          </el-image>
              
                          <div v-if="!attribute.option_hide_text">{{n}}</div>
                        </el-radio>
                      </template>
                    </el-radio-group>
                  </template>
              
                  <!-- Material -->
                  <template v-else-if="$root.isMaterial(attribute)">
                    <el-select v-model="attribute.value"
                      :ref="'input_' + attribute.key"
                      :id="attribute.path + '/' + attribute.key"
                      placeholder="Select"
                      filterable
                      :no-match-text="$root.noMatchMaterialValue(attribute)"
                      no-data-text="No data"
                      :disabled="disabled"
                      class="image-select"
                      :style="{ height: optionImageSize().height }"
                      @change="$emit('change', attribute)">
              
                      <template slot="prefix">
                        <el-image
                          :src="$root.data.materials[attribute.value] ? $root.data.materials[attribute.value].thumbnail : ''"
                          atl="Img" fit="scale-down"
                          class="image-thumbnail material-thumbnail"
                          :class="attribute.option_image_size || 'small'">
                          <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                        </el-image>
                      </template>

                      <template v-if="$root.fullMaterials">
                        <el-option v-for="(name, item) in $root.data.materials"
                          :key="item.value" :label="item.label"
                          :value="item.value">
                          <img :src="item.thumbnail"
                            style="height: 22px;width: 22px;"
                            class="thumbnail-inlist">
                          {{item.label}}
                      </template>

                      <template v-else>
                        <el-option v-for="(v, label) in attribute.options" class="image-option"
                          :key="v" :value="v" :label="label">
              
                          <!-- <template v-for="mat in $root.numMaterials(v)"> -->
                              <!-- <img v-if="$root.data.materials[mat]"
                                :src="$root.data.materials[mat] ? $root.data.materials[mat].thumbnail : ''"
                                style="height: 22px; width:22px;"
                                class="thumbnail-inlist"> -->

                                <el-image v-if="$root.data.materials[v]"
                                  :src="$root.data.materials[v] ? $root.data.materials[v].thumbnail : ''"
                                  atl="Img" fit="scale-down"
                                  :class="(attribute.option_image_size || 'small') + ' image-thumbnail material-thumbnail'">
                                  <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                                </el-image>
                                <span style="padding-left: 5px; color: unset;">{{label}}</span>
                          <!-- </template> -->
                      </template>
                      </el-option>
                    </el-select>
                  </template>
              
                  <!-- default list -->
                  <template v-else>
                    <el-select v-model="attribute.value" 
                      :ref="'input_' + attribute.key"
                      :id="attribute.path + '/' + attribute.key"
                      filterable
                      placeholder=""
                      :disabled="disabled"
                      @change="$emit('change', attribute)" 
                      size="mini"
                      :style="{height: isImageList() ? optionImageSize().height : ''}"
                      :class="attribute.option_use_image ? 'image-select' : ''">
      
                      <template slot="prefix">
                        <template v-if="attribute.option_use_image">
                          <el-image
                            :src="attribute.image_options ? attribute.image_options[attribute.value].base64 : ''"
                            atl="Img" fit="scale-down"
                            :class="attribute.option_image_size + ' image-thumbnail'">
                            <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                          </el-image>
                        </template>
                      </template>
      
                      <el-option v-for="(v, n) in attribute.options" :key="v"
                        :label="n" :value="v" :class="attribute.option_use_image ? 'image-option' : ''">
                        <template v-if="attribute.option_use_image">
                          <el-image
                            :src="attribute.image_options ? attribute.image_options[v].base64 : ''"
                            atl="Img" fit="scale-down"
                            :class="attribute.option_image_size + ' image-thumbnail'">
                            <div slot="error" class="image-error"><i class="fa fa-image fa-2x"></i></div>
                          </el-image>
                        </template>
                        <span style="padding-left: 5px;">{{n}}</span>
                      </el-option>
                    </el-select>
                  </template>
                </template>
              </span>
            </el-tooltip>
          </template>
        </td>
      </tr>
    </table>

  </span>
  `,
  methods: {
    bacgroundImageColor(mat){
      console.log({'background-color': this.$root.data.materials[mat] ? this.$root.data.materials[mat].color : ''})
      return {'background-color': this.$root.data.materials[mat] ? this.$root.data.materials[mat].color : ''}
    },
    querySuggestions(queryString, cb) {
      this.cb_suggestion = cb;
      // return cb([]);
      console.log(queryString, this.attribute.suggestions);

      if (this.attribute.suggestions) {
        cb(
          this.attribute.suggestions.map((value) => {
            return {
              value: value,
              display_value: this.$root.formatDisplayValue(this.attribute, value),
            };
          })
        );
      } else {
        cb([]);
      }
    },
    validationRules(){
      return [
        { 
          validator: (rule, value, callback) => {
            console.log('validator', value)

            if (!value) {
              return callback(new Error('Please input the age'));
            }
            setTimeout(() => {
              if (!Number.isInteger(value)) {
                callback(new Error('Please input digits'));
              } else {
                if (value < 18) {
                  callback(new Error('Age must be greater than 18'));
                } else {
                  callback();
                }
              }
            }, 1000);
          },
          trigger: 'change'
        }
      ]
    },
    getKeyByValue(object, value) {
      return Object.keys(object).find(key => object[key] === value);
    },
    editSelectOption(queryString, cb) {
      console.log(this.attribute.suggestions);
      if (this.attribute.suggestions) {
        cb(
          this.attribute.suggestions.map((value) => {
            return {
              value: value,
            };
          })
        );
      } else {
        cb([]);
      }
    },
    activeValue() {
      return this.attribute.options[this.activeText()];
    },
    activeText() {
      var t;
      if(this.attribute.active_value) {
        t = Object.keys(this.attribute.options).find(l=>{
          return this.attribute.options[l] == this.attribute.active_value;
        })
        if(!t) {
          this.attribute.active_value = Object.values(this.attribute.options)[0];
        }
      }
      if(!t) t = Object.keys(this.attribute.options)[0];

      return t;
    },
    inactiveValue(){
      return this.attribute.options[this.inactiveText()];
    },
    inactiveText(){
      return Object.keys(this.attribute.options).find(l=>{
        return l != this.activeText();
      })
    },
    setElementStyle(){
      // console.log('setElementStyle')
      // console.log(this.$refs)

      const el = this.$refs['input_' + this.attribute.key]
      if (!el) return;
      
      let isRadio = this.attribute.display_type && this.attribute.display_type.toLowerCase() == 'radio';

      if (this.isImageList() && !isRadio) {
        let inputEl = el.$children[0].$el.children[0]
        if (!inputEl) return; 

        if (this.attribute.option_hide_text){
          inputEl.style.color = '#ff000000';
        } else {
          inputEl.style.paddingLeft = parseInt(this.optionImageSize()['height'], 10) + 5 + 'px';
        }
      } 

      // scroll ngang cho radio image
      if(isRadio && this.attribute.set_input_width){
        // console.log('isRadio', el);
        let radioGroupEl = el.$el;

        const labelEl = this.$parent.$children[0].$el

        // find checked el to scroll to
        const checked = Array.from(radioGroupEl.children).find(c => {
          return c.hasAttribute('aria-checked');
        })
        setTimeout(() => {
          if (checked) checked.scrollIntoView({behavior: "smooth", block: "center", inline: "center"});
        }, 300);
      } 
    },
    isImageList(){
      return this.attribute.option_use_image || this.$root.isMaterial(this.attribute);
    },
    validInputSpace(){
      // const el = document.getElementById(this.attribute.path + '/' + this.attribute.key);
      // console.log(el)

      // Find el-form-item__content
      const formItemEl = this.$parent.$el;
      const item__content = Array.from(formItemEl.children).find(c => {
        return c.classList.contains('el-form-item__content');
      })

      console.log(item__content)
      console.log(item__content.offsetWidth)
      return item__content.offsetWidth;
    }
  },
  watch: { 
    value: function(newVal, oldVal){
      if (this.onCancel)
      console.log('value changed: ', oldVal , ' => ', newVal, this.onCancel)

      if (this.onCancel) {
        this.onCancel = false
        display_value = this.$root.convertDisplayValue(this.attribute, oldVal);
        this.attribute.display_value = display_value;
      };
    }, 
    onEdit: function () {
      this.setElementStyle();
    }
  },
  computed: {
    optionImageSize() {
      console.log(this.attribute, this.attribute.option_image_size)

      if(this.attribute.option_image_size){
        return this.optionImageSizes[this.attribute.option_image_size].imageSize;
      } else if (this.$root.isMaterial(this.attribute)){
        return this.optionImageSizes['small'].imageSize;
      } else {
        return 'auto';
      } 
    },
    radioGroupStyle(){
      var style = {}
      if (this.attribute.set_input_width){
        style['width'] = this.attribute.input_width + 'px';
        style['overflow-x'] = 'auto';
        style['padding'] = '2px';
        style['border-radius'] = '4px';
        style['border'] = '1px solid darkgray';
      }
      return style;
    }
  },
  updated: function () {
    this.$nextTick(function () {
      // console.log('updated => setElementStyle')
      this.setElementStyle()
    })
  },
  mounted: function () {
    var self = this;

    // After apply edited attribute
    if (this.attribute) this.setElementStyle();

    if (this.$root.attrCanSetInputWidth(this.attribute) && !this.attribute.on_edit){
      this.attribute.validInputSpace = function () {
        return self.validInputSpace();
      }
    }

    this.$nextTick(() => {
      window.addEventListener("resize", () => {
        // console.log('resize => setElementStyle')
        this.setElementStyle()
      });

      const el = document.getElementById(this.attribute.path + '/' + this.attribute.key);
      
      if (!el) return;

      el.addEventListener("keydown", event => {
        if (event.keyCode == 27){
          // Esc key
          console.log('Esc')
          this.onCancel = true;

          el.blur()
          if (this.cb_suggestion) this.cb_suggestion([]);
        }
      });
    });
  },
});